@@ -1,6 +1,6 @@
 //
 // ActionHandler.cc for pekwm
-// Copyright © 2002-2008 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2009 Claes Nästén <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -15,32 +15,26 @@
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "PMenu.hh"
-#include "PScreen.hh"
+#include "x11.hh"
 #include "Frame.hh"
 #include "Client.hh"
 #include "Config.hh"
 #include "CmdDialog.hh"
+#include "SearchDialog.hh"
 #include "Workspaces.hh"
 #include "WindowManager.hh"
 #include "Util.hh"
 #include "RegexString.hh"
 #include "WorkspaceIndicator.hh"
-
-#ifdef HARBOUR
 #include "Harbour.hh"
-#endif // HARBOUR
-
-#ifdef MENUS
+#include "MenuHandler.hh"
 #include "PDecor.hh"
 #include "PMenu.hh"
 #include "WORefMenu.hh"
 #include "ActionMenu.hh"
 #include "FrameListMenu.hh"
 #include "DecorMenu.hh"
-#ifdef HARBOUR
 #include "HarbourMenu.hh"
-#endif // HARBOUR
-#endif // MENUS
 
 #include <memory>
 
@@ -67,9 +61,9 @@ ActionHandler::ActionHandler(void)
     _instance = this;
 
     // Initialize state_to_keycode map
-    for (uint i = 0; i < PScreen::MODIFIER_TO_MASK_NUM; ++i) {
-        uint modifier = PScreen::MODIFIER_TO_MASK[i];
-        _state_to_keycode[modifier] = PScreen::instance()->getKeycodeFromMask(modifier);
+    for (uint i = 0; i < X11::MODIFIER_TO_MASK_NUM; ++i) {
+        uint modifier = X11::MODIFIER_TO_MASK[i];
+        _state_to_keycode[modifier] = X11::getKeycodeFromMask(modifier);
     }
 }
 
@@ -83,52 +77,35 @@ ActionHandler::~ActionHandler(void)
 void
 ActionHandler::handleAction(const ActionPerformed &ap)
 {
+    PWinObj *wo = ap.wo;
     Client *client = 0;
     Frame *frame = 0;
-#ifdef MENUS
     PMenu *menu = 0;
-#endif // MENUS
     PDecor *decor = 0;
     bool matched = false;
 
-    // determine what type if any of the window object that is focused
-    if (ap.wo) {
-        if (ap.wo->getType() == PWinObj::WO_CLIENT) {
-            client = static_cast<Client*>(ap.wo);
-            frame = static_cast<Frame*>(client->getParent());
-            decor = static_cast<PDecor*>(frame);
-        } else if (ap.wo->getType() == PWinObj::WO_FRAME) {
-            frame = static_cast<Frame*>(ap.wo);
-            client = static_cast<Client*>(frame->getActiveChild());
-            decor = static_cast<PDecor*>(ap.wo);
-#ifdef MENUS
-        } else if (ap.wo->getType() == PWinObj::WO_MENU) {
-            menu = static_cast<PMenu*>(ap.wo);
-            decor = static_cast<PDecor*>(ap.wo);
-#endif // MENUS
-        } else {
-            decor = dynamic_cast<PDecor*>(ap.wo);
-        }
-    }
-
     // go through the list of actions and execute them
     list<Action>::const_iterator it = ap.ae.action_list.begin();
     for (; it != ap.ae.action_list.end(); ++it, matched = false) {
+        // Determine what type if any of the window object that is focused
+        // and check if it is still alive.
+        lookupWindowObjects(&wo, &client, &frame, &menu, &decor);
+
         // actions valid for all PWinObjs
-        if (! matched && ap.wo) {
+        if (! matched && wo) {
             matched = true;
             switch (it->getAction()) {
             case ACTION_FOCUS:
-                ap.wo->giveInputFocus();
+                wo->giveInputFocus();
                 break;
             case ACTION_UNFOCUS:
                 PWinObj::getRootPWinObj()->giveInputFocus();
                 break;
             case ACTION_FOCUS_DIRECTIONAL:
-                actionFocusDirectional(ap.wo, DirectionType(it->getParamI(0)), it->getParamI(1));
+                actionFocusDirectional(wo, DirectionType(it->getParamI(0)), it->getParamI(1));
                 break;
             case ACTION_SEND_KEY:
-                actionSendKey(ap.wo, it->getParamS());
+                actionSendKey(wo, it->getParamS());
                 break;
             default:
                 matched = false;
@@ -149,6 +126,9 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 if ((ap.type == ButtonPress) || (ap.type == ButtonRelease))
                     frame->activateChild(frame->getChildFromPos(ap.event.button->x));
                 break;
+            case ACTION_GOTO_CLIENT:
+                gotoClient(client);
+                break;
             case ACTION_MAXFILL:
                 frame->setStateMaximized(STATE_SET,
                                          it->getParamI(0), it->getParamI(1), true);
@@ -176,14 +156,14 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 break;
             case ACTION_RAISE:
                 if (it->getParamI(0)) {
-                    WindowManager::inst()->familyRaiseLower(client, true);
+                    WindowManager::instance()->familyRaiseLower(client, true);
                 } else {
                     frame->raise();
                 }
                 break;
             case ACTION_LOWER:
                 if (it->getParamI(0)) {
-                    WindowManager::inst()->familyRaiseLower(client, false);
+                    WindowManager::instance()->familyRaiseLower(client, false);
                 } else {
                     frame->lower();
                 }
@@ -198,7 +178,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 if (frame->isFocused()) {
                     frame->raise();
                 } else {
-                    ap.wo->giveInputFocus();
+                    wo->giveInputFocus();
                 }
                 break;
             case ACTION_MOVE_TO_EDGE:
@@ -220,19 +200,22 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 frame->detachClient(client);
                 break;
             case ACTION_ATTACH_MARKED:
-                WindowManager::inst()->attachMarked(frame);
+                WindowManager::instance()->attachMarked(frame);
                 break;
             case ACTION_ATTACH_CLIENT_IN_NEXT_FRAME:
-                WindowManager::inst()->attachInNextPrevFrame(client, false, true);
+                WindowManager::instance()->attachInNextPrevFrame(client, false, true);
                 break;
             case ACTION_ATTACH_CLIENT_IN_PREV_FRAME:
-                WindowManager::inst()->attachInNextPrevFrame(client, false, false);
+                WindowManager::instance()->attachInNextPrevFrame(client, false, false);
                 break;
             case ACTION_ATTACH_FRAME_IN_NEXT_FRAME:
-                WindowManager::inst()->attachInNextPrevFrame(client, true, true);
+                WindowManager::instance()->attachInNextPrevFrame(client, true, true);
                 break;
             case ACTION_ATTACH_FRAME_IN_PREV_FRAME:
-                WindowManager::inst()->attachInNextPrevFrame(client, true, false);
+                WindowManager::instance()->attachInNextPrevFrame(client, true, false);
+                break;
+            case ACTION_SET_OPACITY:
+                actionSetOpacity(client, frame, it->getParamI(0), it->getParamI(1));
                 break;
             default:
                 matched = false;
@@ -241,7 +224,6 @@ ActionHandler::handleAction(const ActionPerformed &ap)
         }
 
         // Actions valid for Menus
-#ifdef MENUS
         if (! matched && menu) {
             matched = true;
             switch (it->getAction()) {
@@ -271,14 +253,13 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 break;
             case ACTION_CLOSE:
                 menu->unmapAll();
-                WindowManager::inst()->findWOAndFocus(0);
+                WindowManager::instance()->findWOAndFocus(0);
                 break;
             default:
                 matched = false;
                 break;
             }
         }
-#endif // MENUS
         // actions valid for pdecor
         if (! matched && decor) {
             int x_root, y_root;
@@ -289,14 +270,15 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 // Get root position, previously event positions was
                 // used however this seems to be error prone on
                 // Xinerama setups
-                PScreen::instance()->getMousePosition(x_root, y_root);
+                X11::getMousePosition(x_root, y_root);
 
                 decor->doMove(x_root, y_root);
                 break;
             case ACTION_CLOSE:
                 decor->unmapWindow();
-                if (decor->getType() == PWinObj::WO_CMD_DIALOG) {
-                    WindowManager::inst()->findWOAndFocus(0);
+                if (decor->getType() == PWinObj::WO_CMD_DIALOG
+                    || decor->getType() == PWinObj::WO_SEARCH_DIALOG) {
+                    WindowManager::instance()->findWOAndFocus(0);
                 }
                 break;
             case ACTION_WARP_TO_WORKSPACE:
@@ -315,7 +297,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
             case ACTION_SET:
             case ACTION_UNSET:
             case ACTION_TOGGLE:
-                handleStateAction(*it, ap.wo, client, frame);
+                handleStateAction(*it, wo, client, frame);
                 break;
             case ACTION_NEXT_FRAME:
                 actionFocusToggle(ap.ae.sym, it->getParamI(0), 1 /* Direction */,
@@ -346,41 +328,45 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 actionGotoClientID(it->getParamI(0));
                 break;
             case ACTION_EXEC:
-                if (it->getParamS().size())
-                    Util::forkExec(it->getParamS());
+                actionExec(client, it->getParamS());
                 break;
-#ifdef MENUS
             case ACTION_SHOW_MENU:
                 actionShowMenu(it->getParamS(), it->getParamI(0),
-                               ap.type, client ? client : ap.wo);
+                               ap.type, client ? client : wo);
                 break;
             case ACTION_HIDE_ALL_MENUS:
-                WindowManager::inst()->hideAllMenus();
+                MenuHandler::instance()->hideAllMenus();
                 break;
-#endif // MENUS
             case ACTION_RELOAD:
-                WindowManager::inst()->reload();
+                WindowManager::instance()->reload();
                 break;
             case ACTION_RESTART:
-                WindowManager::inst()->restart();
+                WindowManager::instance()->restart();
                 break;
             case ACTION_RESTART_OTHER:
                 if (it->getParamS().size())
-                    WindowManager::inst()->restart(it->getParamS());
+                    WindowManager::instance()->restart(it->getParamS());
                 break;
             case ACTION_EXIT:
-                WindowManager::inst()->shutdown();
+                WindowManager::instance()->shutdown();
                 break;
             case ACTION_SHOW_CMD_DIALOG:
-                if (WindowManager::inst()->getCmdDialog()->isMapped()) {
-                    WindowManager::inst()->getCmdDialog()->unmapWindow();
+                if (WindowManager::instance()->getCmdDialog()->isMapped()) {
+                    WindowManager::instance()->getCmdDialog()->unmapWindow();
                 } else {
-                    WindowManager::inst()->getCmdDialog()->mapCentered(it->getParamS(), true,
-                                                     frame ? frame : ap.wo);
+                    WindowManager::instance()->getCmdDialog()->mapCentered(it->getParamS(), true,
+                                                     frame ? frame : wo);
                 }
                 break;
+            case ACTION_SHOW_SEARCH_DIALOG:
+                if (WindowManager::instance()->getSearchDialog()->isMapped()) {
+                    WindowManager::instance()->getSearchDialog()->unmapWindow();
+                } else {
+                    WindowManager::instance()->getSearchDialog()->mapCentered(it->getParamS(), true, frame ? frame : wo);
+                } 
+                break;
             case ACTION_HIDE_WORKSPACE_INDICATOR:
-                WindowManager::inst()->getWorkspaceIndicator()->unmapWindow();
+                WindowManager::instance()->getWorkspaceIndicator()->unmapWindow();
                 break;
             default:
                 matched = false;
@@ -390,6 +376,35 @@ ActionHandler::handleAction(const ActionPerformed &ap)
     }
 }
 
+void
+ActionHandler::lookupWindowObjects(PWinObj **wo, Client **client, Frame **frame,
+                                   PMenu **menu, PDecor **decor)
+{
+    *client = 0;
+    *frame = 0;
+    *menu = 0;
+    *decor = 0;
+
+    if (PWinObj::windowObjectExists(*wo)) {
+        if ((*wo)->getType() == PWinObj::WO_CLIENT) {
+            *client = static_cast<Client*>(*wo);
+            *frame = static_cast<Frame*>((*client)->getParent());
+            *decor = static_cast<PDecor*>(*frame);
+        } else if ((*wo)->getType() == PWinObj::WO_FRAME) {
+            *frame = static_cast<Frame*>(*wo);
+            *client = static_cast<Client*>((*frame)->getActiveChild());
+            *decor = static_cast<PDecor*>(*wo);
+        } else if ((*wo)->getType() == PWinObj::WO_MENU) {
+            *menu = static_cast<PMenu*>(*wo);
+            *decor = static_cast<PDecor*>(*wo);
+        } else {
+            *decor = dynamic_cast<PDecor*>(*wo);
+        }
+    } else {
+        *wo = 0;
+    }
+}
+
 //! @brief Handles state actions
 void
 ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
@@ -449,6 +464,9 @@ ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
         case ACTION_STATE_TITLE:
             frame->setStateTitle(sa, client, Util::to_wide_str(action.getParamS()));
             break;
+        case ACTION_STATE_OPAQUE:
+            frame->setStateOpaque(sa);
+            break;
         default:
             matched = false;
             break;
@@ -472,13 +490,11 @@ ActionHandler::handleStateAction(const Action &action, PWinObj *wo,
     if (! matched) {
         matched = true;
         switch (action.getParamI(0)) {
-#ifdef HARBOUR
         case ACTION_STATE_HARBOUR_HIDDEN:
-            WindowManager::inst()->getHarbour()->setStateHidden(sa);
+            WindowManager::instance()->getHarbour()->setStateHidden(sa);
             break;
-#endif // HARBOUR
         case ACTION_STATE_GLOBAL_GROUPING:
-            WindowManager::inst()->setStateGlobalGrouping(sa);
+            WindowManager::instance()->setStateGlobalGrouping(sa);
             break;
         default:
             matched = false;
@@ -506,8 +522,8 @@ ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, std
         return 0;
     }
 
-    state &= ~PScreen::instance()->getNumLock() & ~PScreen::instance()->getScrollLock() & ~LockMask;
-    state &= ~Button1Mask & ~Button2Mask & ~Button3Mask & ~Button4Mask & ~Button5Mask;
+    X11::stripStateModifiers(&state);
+    X11::stripButtonModifiers(&state);
 
     list<ActionEvent>::iterator it(actions->begin());
     for (; it != actions->end(); ++it) {
@@ -521,6 +537,18 @@ ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, std
     return 0;
 }
 
+/**
+ * Execute action, setting client environment before (if any).
+ */
+void
+ActionHandler::actionExec(Client *client, const std::string &command)
+{
+    if (command.size()) {
+        Client::setClientEnvironment(client);
+        Util::forkExec(command);
+    }
+}
+
 //! @brief Searches for a client matching titles and makes it visible
 void
 ActionHandler::actionFindClient(const std::wstring &title)
@@ -537,14 +565,7 @@ ActionHandler::actionFindClient(const std::wstring &title)
 void
 ActionHandler::actionGotoWorkspace(uint workspace, bool warp)
 {
-    Workspaces::instance()->gotoWorkspace(workspace, warp);
-
-    if (Config::instance()->getShowWorkspaceIndicator() > 0) {
-        // Show workspace indicator if requested
-        WindowManager::inst()->getWorkspaceIndicator()->render();
-        WindowManager::inst()->getWorkspaceIndicator()->mapWindowRaised();
-        WindowManager::inst()->getWorkspaceIndicator()->updateHideTimer(Config::instance()->getShowWorkspaceIndicator());
-    }
+    Workspaces::gotoWorkspace(workspace, warp);
 }
 
 //! @brief Focus client with id.
@@ -563,27 +584,50 @@ ActionHandler::actionGotoClientID(uint id)
 void
 ActionHandler::actionSendToWorkspace(PDecor *decor, int direction)
 {
-    // FIXME: Add Up, Down, PrevV and NextV support
+    // Convenience
+    const uint per_row = Workspaces::getPerRow(),
+        cur_row = Workspaces::getRow(),
+        cur_act = Workspaces::getActive(),
+        row_min = Workspaces::getRowMin(),
+        row_max = Workspaces::getRowMax();
+
     switch (direction) {
     case WORKSPACE_LEFT:
     case WORKSPACE_PREV:
-        if (Workspaces::instance()->getActive() > 0) {
-            decor->setWorkspace(Workspaces::instance()->getActive() - 1);
+        if (cur_act > row_min) {
+            decor->setWorkspace(cur_act - 1);
         } else if (static_cast<uint>(direction) == WORKSPACE_PREV) {
-            decor->setWorkspace(Workspaces::instance()->size() - 1);
+            decor->setWorkspace(row_max);
         }
         break;
     case WORKSPACE_NEXT:
     case WORKSPACE_RIGHT:
-        if ((Workspaces::instance()->getActive() + 1) <
-                Workspaces::instance()->size()) {
-            decor->setWorkspace(Workspaces::instance()->getActive() + 1);
+        if (cur_act < row_max) {
+            decor->setWorkspace(cur_act + 1);
         } else if (static_cast<uint>(direction) == WORKSPACE_NEXT) {
-            decor->setWorkspace(0);
+            decor->setWorkspace(row_min);
+        }
+        break;
+    case WORKSPACE_PREV_V:
+    case WORKSPACE_UP:
+        if (cur_act >= per_row) {
+            decor->setWorkspace(cur_act - per_row);
+        } else if (static_cast<uint>(direction) == WORKSPACE_PREV_V) {
+            // Bottom left + column
+            decor->setWorkspace(Workspaces::size() - per_row
+                                + cur_act - cur_row * per_row);
+        }
+        break;
+    case WORKSPACE_NEXT_V:
+    case WORKSPACE_DOWN:
+        if ((cur_act + per_row) < Workspaces::size()) {
+            decor->setWorkspace(cur_act + per_row);
+        } else if (static_cast<uint>(direction) == WORKSPACE_NEXT_V) {
+            decor->setWorkspace(cur_act - cur_row * per_row);
         }
         break;
     case WORKSPACE_LAST:
-        decor->setWorkspace(Workspaces::instance()->getPrevious());
+        decor->setWorkspace(Workspaces::getPrevious());
         break;
     default:
         decor->setWorkspace(direction);
@@ -595,14 +639,18 @@ ActionHandler::actionSendToWorkspace(PDecor *decor, int direction)
 void
 ActionHandler::actionWarpToWorkspace(PDecor *decor, uint direction)
 {
+    // Removing the already accumulated motion events can help
+    // to avoid skipping workspaces (see task #77).
+    X11::removeMotionEvents();
+
     // actually did move
-    if (Workspaces::instance()->gotoWorkspace(DirectionType(direction), true)) {
+    if (Workspaces::gotoWorkspace(DirectionType(direction), true)) {
         int x, y;
-        PScreen::instance()->getMousePosition(x, y);
+        X11::getMousePosition(x, y);
 
         decor->move(decor->getClickX() + x - decor->getPointerX(),
                     decor->getClickY() + y - decor->getPointerY());
-        decor->setWorkspace(Workspaces::instance()->getActive());
+        decor->setWorkspace(Workspaces::getActive());
     }
 }
 
@@ -627,40 +675,30 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
     }
 
     // unable to grab keyboard
-    if (! PScreen::instance()->grabKeyboard(PScreen::instance()->getRoot())) {
+    if (! X11::grabKeyboard(X11::getRoot())) {
         return;
     }
 
     // find the focused window object
     PWinObj *fo_wo = 0;
-    if (PWinObj::getFocusedPWinObj()) {
-        if (PWinObj::getFocusedPWinObj()->getType() == PWinObj::WO_CLIENT) {
-            fo_wo = PWinObj::getFocusedPWinObj()->getParent();
+    if (PWinObj::isFocusedPWinObj(PWinObj::WO_CLIENT)) {
+        fo_wo = PWinObj::getFocusedPWinObj()->getParent();
 
-            list<PMenu::Item*>::iterator it(menu->m_begin());
-            for (; it != menu->m_end(); ++it) {
-                if ((*it)->getWORef() == fo_wo) {
-                    menu->selectItem(it);
-                    break;
-                }
+        list<PMenu::Item*>::iterator it(menu->m_begin());
+        for (; it != menu->m_end(); ++it) {
+            if ((*it)->getWORef() == fo_wo) {
+                menu->selectItem(it);
+                break;
             }
-            fo_wo->setFocused(false);
-
-        } else {
-#ifdef DEBUG
-            cerr << __FILE__ << "@" << __LINE__ << ": "
-                 << "ActionHandler(" << this << ")::actionFocusToggle("
-                 << button << "," << raise << "," << off << "," << mru << ")"
-                 << endl << " *** focused PWinObj != WO_FRAME" << endl;
-#endif // DEBUG
         }
+        fo_wo->setFocused(false);
     }
 
     if (Config::instance()->getShowFrameList()) {
         menu->buildMenu();
 
         Geometry head;
-        PScreen::instance()->getHeadInfo(PScreen::instance()->getCurrHead(), head);
+        X11::getHeadInfo(X11::getCurrHead(), head);
         menu->move(head.x + ((head.width - menu->getWidth()) / 2),
                    head.y + ((head.height - menu->getHeight()) / 2));
         menu->setFocused(true);
@@ -687,7 +725,7 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
             }
         }
 
-        XMaskEvent(PScreen::instance()->getDpy(), KeyPressMask|KeyReleaseMask, &ev);
+        XMaskEvent(X11::getDpy(), KeyPressMask|KeyReleaseMask, &ev);
         if (ev.type == KeyPress) {
             if (ev.xkey.keycode == button) {
                 if (fo_wo) {
@@ -702,27 +740,27 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
                 menu->selectItemRel(off);
                 fo_wo = menu->getItemCurr()->getWORef();
             } else {
-                XPutBackEvent (PScreen::instance()->getDpy(), &ev);
+                XPutBackEvent(X11::getDpy(), &ev);
                 cycling = false;
             }
         } else if (ev.type == KeyRelease) {
-            if (IsModifierKey(XKeycodeToKeysym(PScreen::instance()->getDpy(),
+            if (IsModifierKey(XKeycodeToKeysym(X11::getDpy(),
                                                ev.xkey.keycode, 0))) {
                 cycling = false;
             }
         } else {
-            XPutBackEvent(PScreen::instance()->getDpy(), &ev);
+            XPutBackEvent(X11::getDpy(), &ev);
         }
     }
 
-    PScreen::instance()->ungrabKeyboard();
+    X11::ungrabKeyboard();
 
     // Got something to focus
     if (fo_wo) {
         // De-iconify if iconified, user probably wants this
         if (fo_wo->isIconified()) {
             // If the window was iconfied, and sticky
-            fo_wo->setWorkspace(Workspaces::instance()->getActive());
+            fo_wo->setWorkspace(Workspaces::getActive());
             fo_wo->mapWindow();
             fo_wo->raise();
         } else if (Raise(raise) == END_RAISE) {
@@ -740,7 +778,7 @@ void
 ActionHandler::actionFocusDirectional(PWinObj *wo, DirectionType dir, bool raise)
 {
     PWinObj *wo_focus =
-        Workspaces::instance()->findDirectional(wo, dir, SKIP_FOCUS_TOGGLE);
+        Workspaces::findDirectional(wo, dir, SKIP_FOCUS_TOGGLE);
 
     if (wo_focus) {
         if (raise) {
@@ -777,22 +815,22 @@ ActionHandler::actionSendKey(PWinObj *wo, const std::string &key_str)
     for (it = _state_to_keycode.begin(); it != _state_to_keycode.end(); ++it) {
         if (mod & it->first) {
             ev.xkey.keycode = it->second;
-            XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+            XSendEvent(X11::getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
             ev.xkey.state |= it->first;
         }
     }
 
     // Send press and release of main key
     ev.xkey.keycode = key;
-    XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+    XSendEvent(X11::getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
     ev.type = KeyRelease;
-    XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+    XSendEvent(X11::getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
 
     // Release state modifiers
     for (it = _state_to_keycode.begin(); it != _state_to_keycode.end(); ++it) {
         if (mod & it->first) {
             ev.xkey.keycode = it->second;
-            XSendEvent(PScreen::instance()->getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
+            XSendEvent(X11::getDpy(), wo->getWindow(), True, KeyPressMask, &ev);
             ev.xkey.state &= ~it->first;
         }
     }
@@ -800,7 +838,18 @@ ActionHandler::actionSendKey(PWinObj *wo, const std::string &key_str)
     return true;
 }
 
-#ifdef MENUS
+void
+ActionHandler::actionSetOpacity(PWinObj *client, PWinObj *frame, uint focus, uint unfocus)
+{
+    if (! unfocus) {
+        unfocus = focus;
+    }
+    CONV_OPACITY(focus);
+    CONV_OPACITY(unfocus);
+    client->setOpacity(focus, unfocus);
+    frame->setOpacity(client);
+}
+
 //! @brief Toggles visibility of menu.
 //! @param name Name of menu to toggle visibilty of
 //! @param stick Stick menu when showing
@@ -810,7 +859,7 @@ void
 ActionHandler::actionShowMenu(const std::string &name, bool stick,
                               uint e_type, PWinObj *wo_ref)
 {
-    PMenu *menu = WindowManager::inst()->getMenu(name);
+    PMenu *menu = MenuHandler::instance()->getMenu(name);
     if (! menu) {
         return;
     }
@@ -843,7 +892,6 @@ ActionHandler::actionShowMenu(const std::string &name, bool stick,
         }
     }
 }
-#endif // MENUS
 
 //! @brief Creates a menu containing a list of Frames currently visible
 //! @param show_iconified Flag to show/hide iconified windows
@@ -852,13 +900,13 @@ ActionHandler::createNextPrevMenu(bool show_iconified)
 {
     ActionEvent ae; // empty ae, used when inserting
     PMenu *menu =
-        new PMenu(PScreen::instance()->getDpy(), WindowManager::inst()->getTheme(), L"Windows",
+        new PMenu(WindowManager::instance()->getTheme(), L"Windows",
                   "" /* Empty name*/);
 
     Frame *fr;
     list<Frame*>::iterator f_it(Frame::frame_begin());
     for (; f_it != Frame::frame_end(); ++f_it) {
-        fr = static_cast<Frame*>(*f_it);
+        fr = *f_it;
         if (createMenuInclude(fr, show_iconified)) {
             menu->insert(static_cast<Client*>(fr->getActiveChild())->getTitle()->getVisible(),
                          ae, fr, static_cast<Client*>(fr->getActiveChild())->getIcon());
@@ -874,12 +922,12 @@ PMenu*
 ActionHandler::createMRUMenu(bool show_iconified)
 {
     ActionEvent ae; // empty ae, used when inserting
-    PMenu *menu = new PMenu(PScreen::instance()->getDpy(),
-            WindowManager::inst()->getTheme(), L"MRU Windows", "" /* Empty name */);
+    PMenu *menu = new PMenu(WindowManager::instance()->getTheme(),
+                            L"MRU Windows", "" /* Empty name */);
 
     Frame *fr;
-    list<PWinObj*>::reverse_iterator f_it = WindowManager::inst()->mru_rbegin();
-    for (; f_it != WindowManager::inst()->mru_rend(); ++f_it) {
+    list<Frame*>::reverse_iterator f_it = WindowManager::instance()->mru_rbegin();
+    for (; f_it != WindowManager::instance()->mru_rend(); ++f_it) {
         fr = static_cast<Frame*>(*f_it);
         if (createMenuInclude(fr, show_iconified)) {
             menu->insert(static_cast<Client*>(fr->getActiveChild())->getTitle()->getVisible(),
@@ -903,7 +951,7 @@ ActionHandler::createMenuInclude(Frame *frame, bool show_iconified)
     // we split it up for readability.
     
     // focw == frame on current workspace
-    bool focw = frame->isSticky() || frame->getWorkspace() == Workspaces::instance()->getActive();
+    bool focw = frame->isSticky() || frame->getWorkspace() == Workspaces::getActive();
     
     // ibs == iconified but should be shown
     bool ibs = show_iconified && frame->isIconified() && focw;
@@ -949,8 +997,8 @@ ActionHandler::gotoClient(Client *client)
 
     // Make sure it's visible
     if (! frame->isSticky()
-        && (frame->getWorkspace() != Workspaces::instance()->getActive())) {
-        Workspaces::instance()->setWorkspace(frame->getWorkspace(), false);
+        && (frame->getWorkspace() != Workspaces::getActive())) {
+        Workspaces::setWorkspace(frame->getWorkspace(), false);
     }
 
     if (! frame->isMapped()) {
@@ -973,13 +1021,13 @@ void
 ActionHandler::initSendKeyEvent(XEvent &ev, PWinObj *wo)
 {
     ev.type = KeyPress;
-    ev.xkey.display = PScreen::instance()->getDpy();
-    ev.xkey.root = PScreen::instance()->getRoot();
+    ev.xkey.display = X11::getDpy();
+    ev.xkey.root = X11::getRoot();
     ev.xkey.window = wo->getWindow();
     ev.xkey.time = CurrentTime;
     ev.xkey.x = 1;
     ev.xkey.y = 1;
-    PScreen::instance()->getMousePosition(ev.xkey.x_root, ev.xkey.y_root);
+    X11::getMousePosition(ev.xkey.x_root, ev.xkey.y_root);
     ev.xkey.same_screen = True;
     ev.xkey.type = KeyPress;
     ev.xkey.state = 0;
