@@ -74,9 +74,8 @@ AutoProperties::AutoProperties(void)
     _property_map["CFGDENY"] = AP_CFG_DENY;
     _property_map["ALLOWEDACTIONS"] = AP_ALLOWED_ACTIONS;
     _property_map["DISALLOWEDACTIONS"] = AP_DISALLOWED_ACTIONS;
-#ifdef OPACITY
     _property_map["OPACITY"] = AP_OPACITY;
-#endif // OPACITY
+    _property_map["DECOR"] = AP_DECOR;
 
     // group properties
     _group_property_map[""] = AP_NO_PROPERTY;
@@ -260,7 +259,7 @@ AutoProperties::findProperty(const ClassHint* class_hint,
     list<uint>::iterator w_it;
 
     // start searching for a suitable property
-    for (bool ok = false; it != prop_list->end(); ++it, ok = false) {
+    for (; it != prop_list->end(); ++it) {
         // see if the type matches, if we have one
         if ((type != 0) && ! (*it)->isApplyOn(type))
             continue;
@@ -820,12 +819,14 @@ AutoProperties::parseAutoPropertyValue(CfgParser::Entry *section, AutoProperty *
             prop->maskAdd(AP_DISALLOWED_ACTIONS);
             Config::instance()->parseActionAccessMask((*it)->get_value(), prop->disallowed_actions);
             break;
-#ifdef OPACITY
         case AP_OPACITY:
             prop->maskAdd(AP_OPACITY);
             Config::parseOpacity((*it)->get_value(), prop->focus_opacity, prop->unfocus_opacity);
             break;
-#endif // OPACITY
+        case AP_DECOR:
+            prop->maskAdd(AP_DECOR);
+            prop->frame_decor = (*it)->get_value();
+            break;
         default:
             // do nothing
             break;
