@@ -1,5 +1,5 @@
 //
-// Copyright © 2005-2008 Claes Nästén <me@pekdon.net>
+// Copyright © 2005-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -231,7 +231,7 @@ operator<<(std::ostream &stream, const CfgParser::Entry &entry)
 
 //! @brief CfgParser constructor.
 CfgParser::CfgParser(void)
-    : _source(0), _root_entry(0),
+    : _source(0), _root_entry(0), _is_dynamic_content(false),
       _section(_root_entry), _overwrite(false)
 {
     _root_entry = new CfgParser::Entry(_root_source_name, 0, "ROOT", "");
@@ -306,6 +306,9 @@ CfgParser::parse(const std::string &src, CfgParserSource::Type type, bool overwr
     int c, next;
     while (_source_list.size()) {
         _source = _source_list.back();
+        if (_source->is_dynamic()) {
+            _is_dynamic_content = true;
+        }
 
         while ((c = _source->getc()) != EOF) {
             switch (c) {
@@ -399,6 +402,11 @@ CfgParser::parse_source_new(const std::string &name_orig, CfgParserSource::Type
         // Open and set as active, delete if fails.
         try {
             source->open();
+            // Add source to file list if file
+            if (type == CfgParserSource::SOURCE_FILE) {
+                _file_list[name] = Util::getMtime(name);
+            }
+
             _source = source;
             _source_list.push_back(_source);
             done = 1;
@@ -457,64 +465,41 @@ CfgParser::parse_name(std::string &buf)
 }
 
 //! @brief Parses from = to end of " pair.
-bool
+void
 CfgParser::parse_value(CfgParserSource *source, std::string &value)
 {
-    // Init parse buffer and reserve memory.
-    string buf;
-    buf.reserve(PARSE_BUF_SIZE);
-
-    // We expect to get a " after the =, however to do proper error reporting
-    // we store the what we get between so we can show the output if it includes
-    // anything else than spaces.
-    int c, next;
-    bool garbage = false;
-    while ((c = source->getc()) != EOF) {
-        if (c == '"') {
-            break;
-        }
-
-        buf += c;
-
-        if (! isspace(c)) {
-            garbage = true;
-        }
-    }
+    // We expect to get a " after the =, however we ignore anything else.
+    int c;
+    while ((c = source->getc()) != EOF && c != '"')
+         ;
 
     // Check if we got to a " or found EOF first.
     if (c == EOF) {
         cerr << "Reached EOF before opening \" in value." << endl;
-        return false;
-    }
-
-    // Check if there was garbage between = and ".
-    if (garbage) {
-        // pass, do nothing
+        return;
     }
 
     // Parse until next ", and escape characters after \.
-    buf.clear();
-    while ((c = source->getc()) != EOF) {
+    while ((c = source->getc()) != EOF && c != '"') {
         // Escape character after \, if newline drop it.
         if (c == '\\') {
-            next = source->getc();
-            if (next != '\n') {
-                buf += next;
+            c = source->getc();
+            if (c == '\n' || c == EOF) {
+                continue;
             }
-        } else if (c == '"') {
-            break;
-        } else {
-            buf += c;
         }
+        value += c;
     }
 
     if (c == EOF) {
         cerr << "Reached EOF before closing \" in value." << endl;
     }
 
-    value = buf;
-
-    return false;
+    // If the value is empty, parse_entry_finish() might later just skip
+    // the complete entry. To allow empty config options we add a dummy space.
+    if (!value.size()) {
+    	value = " ";
+    }
 }
 
 //! @brief Parses entry (name + value) and executes command accordingly.
@@ -627,8 +612,12 @@ CfgParser::parse_comment_c(CfgParserSource *source)
 {
     int c;
     while ((c = source->getc()) != EOF) {
-        if ((c == '*') && (source->getc() == '/')) {
-            break;
+        if (c == '*') {
+            if ((c = source->getc()) == '/') {
+                break;
+        	} else if (c != EOF) {
+                source->ungetc(c);
+            }
         }
     }
 
@@ -639,7 +628,7 @@ CfgParser::parse_comment_c(CfgParserSource *source)
 
 //! @brief Parses Source until next non whitespace char is found.
 char
-CfgParser::parse_skip_blank (CfgParserSource *source)
+CfgParser::parse_skip_blank(CfgParserSource *source)
 {
     int c;
     while (((c = source->getc()) != EOF) && isspace(c))
@@ -689,42 +678,60 @@ CfgParser::variable_define(const std::string &name, const std::string &value)
 void
 CfgParser::variable_expand(std::string &var)
 {
-    string::size_type begin = 0, end = 0;
+    bool did_expand;
 
-    while ((begin = var.find_first_of('$', end)) != string::npos) {
-        end = begin + 1;
+    do {
+        did_expand = false;
 
-        // Skip escaped \$
-        if ((begin > 0) && (var[begin - 1] == '\\')) {
-            continue;
-        }
+        string::size_type begin = 0, end = 0;
+        while ((begin = var.find_first_of('$', end)) != string::npos) {
+            end = begin + 1;
 
-	// Find end of variable
-        for (; end != var.size(); ++end) {
-            if ((isalnum(var[end]) == 0) && (var[end] != '_'))  {
-                break;
+            // Skip escaped \$
+            if ((begin > 0) && (var[begin - 1] == '\\')) {
+                continue;
             }
-        }
 
-        string var_name(var.substr(begin, end - begin));
-        // If the variable starts with _ it is considered an environment
-        // variable, use getenv to see if it is available
-        if (var_name.size() > 2 && var_name[1] == '_') {
-            char *value = getenv(var_name.c_str() + 2);
-            if (value) {
-                var.replace(begin, end - begin, value);
-                end = begin + strlen(value);
-            } else {
-                cerr << "Trying to use undefined environment variable: " << var_name << endl;;
+            // Find end of variable
+            for (; end != var.size(); ++end) {
+                if ((isalnum(var[end]) == 0) && (var[end] != '_'))  {
+                    break;
+                }
             }
+
+            did_expand = variable_expand_name(var, begin, end) || did_expand;
+        }
+    } while (did_expand);
+}
+
+bool
+CfgParser::variable_expand_name(std::string &var,
+                                string::size_type begin, string::size_type &end)
+{
+    bool did_expand = false;
+    string var_name(var.substr(begin, end - begin));
+
+    // If the variable starts with _ it is considered an environment
+    // variable, use getenv to see if it is available
+    if (var_name.size() > 2 && var_name[1] == '_') {
+        char *value = getenv(var_name.c_str() + 2);
+        if (value) {
+            var.replace(begin, end - begin, value);
+            end = begin + strlen(value);
+            did_expand = true;
         } else {
-            map<string, string>::iterator it(_var_map.find(var_name));
-            if (it != _var_map.end()) {
-                var.replace(begin, end - begin, it->second);
-                end = begin + it->second.size();
-            } else  {
-                cerr << "Trying to use undefined variable: " << var_name << endl;
-            }
+            cerr << "Trying to use undefined environment variable: " << var_name << endl;;
+        }
+    } else {
+        map<string, string>::iterator it(_var_map.find(var_name));
+        if (it != _var_map.end()) {
+            var.replace(begin, end - begin, it->second);
+            end = begin + it->second.size();
+            did_expand = true;
+        } else  {
+            cerr << "Trying to use undefined variable: " << var_name << endl;
         }
     }
+
+    return did_expand;
 }
