@@ -30,7 +30,7 @@ extern "C" {
 #include "PWinObj.hh"
 #include "PDecor.hh" // PDecor::TitleItem
 #include "Client.hh"
-#include "PScreen.hh"
+#include "x11.hh"
 #include "Config.hh"
 #include "KeyGrabber.hh"
 #include "Theme.hh"
@@ -61,19 +61,21 @@ Client::Client(Window new_client, bool is_new)
       _id(0), _size(0),
       _transient(None), _strut(0), _icon(0),
       _pid(0), _is_remote(false), _class_hint(0),
+      _window_type(WINDOW_TYPE_NORMAL),
       _alive(false), _marked(false),
       _send_focus_message(false), _send_close_message(false),
       _wm_hints_input(true), _cfg_request_lock(false),
-      _shaped(false), _extended_net_name(false)
+      _shaped(false), _extended_net_name(false),
+      _demands_attention(false)
 {
     // PWinObj attributes, required by validate etc.
     _window = new_client;
     _type = WO_CLIENT;
 
     // Construct the client
-    PScreen::instance()->grabServer();
+    X11::grabServer();
     if (! validate() || ! getAndUpdateWindowAttributes()) {
-        PScreen::instance()->ungrabServer(true);
+        X11::ungrabServer(true);
         return;
     }
 
@@ -82,14 +84,14 @@ Client::Client(Window new_client, bool is_new)
     _title.setId(_id);
     _title.infoAdd(PDecor::TitleItem::INFO_ID);
 
-    if (PScreen::instance()->hasExtensionShape()) {
+    if (X11::hasExtensionShape()) {
 #ifdef HAVE_SHAPE
-        XShapeSelectInput(_dpy, _window, ShapeNotifyMask);
+        XShapeSelectInput(X11::getDpy(), _window, ShapeNotifyMask);
 #endif // HAVE_SHAPE
     }
 
-    XAddToSaveSet(_dpy, _window);
-    XSetWindowBorderWidth(_dpy, _window, 0);
+    XAddToSaveSet(X11::getDpy(), _window);
+    XSetWindowBorderWidth(X11::getDpy(), _window, 0);
 
     // Load the Class hint before loading the autoprops and
     // getting client title as we search for TitleRule in the autoprops
@@ -126,7 +128,7 @@ Client::Client(Window new_client, bool is_new)
     // Tell the world about our state
     updateEwmhStates();
 
-    PScreen::instance()->ungrabServer(true);
+    X11::ungrabServer(true);
 
     setMappedStateAndFocus(is_new, ap);
 
@@ -154,7 +156,7 @@ Client::~Client(void)
 
     returnClientID(_id);
 
-    PScreen::instance()->grabServer();
+    X11::grabServer();
 
     // removes gravity and moves it back to root if we are alive
     bool focus = false;
@@ -174,9 +176,9 @@ Client::~Client(void)
     // Clean up if the client still is alive, it'll be dead all times
     // except when we exit pekwm
     if (_alive) {
-        XUngrabButton(_dpy, AnyButton, AnyModifier, _window);
+        X11::ungrabButton(_window);
         KeyGrabber::instance()->ungrabKeys(_window);
-        XRemoveFromSaveSet(_dpy, _window);
+        XRemoveFromSaveSet(X11::getDpy(), _window);
         PWinObj::mapWindow();
     }
 
@@ -196,7 +198,7 @@ Client::~Client(void)
         _icon = 0;
     }
 
-    PScreen::instance()->ungrabServer(true);
+    X11::ungrabServer(true);
 }
 
 /**
@@ -208,7 +210,7 @@ bool
 Client::getAndUpdateWindowAttributes(void)
 {
     XWindowAttributes attr;
-    if (! XGetWindowAttributes(_dpy, _window, &attr)) {
+    if (! XGetWindowAttributes(X11::getDpy(), _window, &attr)) {
         return false;
     }
     _gm.x = attr.x;
@@ -226,7 +228,8 @@ Client::getAndUpdateWindowAttributes(void)
         ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
 
     // We don't want these masks to be propagated down to the frame
-    XChangeWindowAttributes(_dpy, _window, CWEventMask|CWDontPropagate, &sattr);
+    XChangeWindowAttributes(X11::getDpy(), _window,
+                            CWEventMask|CWDontPropagate, &sattr);
 
     return true;
 }
@@ -341,7 +344,7 @@ void
 Client::setInitialState(void)
 {
     // Set state either specified in hint
-    ulong initial_state = readWmHints();
+    ulong initial_state = getWMHints();
     if (getWmState() == IconicState) {
         _iconified = true;
     }
@@ -403,8 +406,7 @@ Client::findAndRaiseIfTransient(void)
         Frame *frame = static_cast<Frame*>(getParent());
         Frame *frame_transient = static_cast<Frame*>(_transient->getParent());
         if (frame->getActiveChild() == this) {
-            frame->setLayer(frame_transient->getLayer() + 1);
-            frame->raise();
+            Workspaces::stackAbove(this, frame_transient->getWindow());
         }
     }
 }
@@ -430,10 +432,10 @@ Client::mapWindow(void)
         mapOrUnmapTransients(_window, false);
     }
 
-    XSelectInput(_dpy, _window, NoEventMask);
+    X11::selectInput(_window, NoEventMask);
     PWinObj::mapWindow();
-    XSelectInput(_dpy, _window,
-                 PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
+    X11::selectInput(_window,
+                     PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
 }
 
 
@@ -451,9 +453,10 @@ Client::unmapWindow(void)
         updateEwmhStates();
     }
 
-    XSelectInput(_dpy, _window, NoEventMask);
+    X11::selectInput(_window, NoEventMask);
     PWinObj::unmapWindow();
-    XSelectInput(_dpy, _window, PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
+    X11::selectInput(_window,
+                         PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
 }
 
 //! @brief Iconifies the client and adds it to the iconmenu
@@ -529,8 +532,8 @@ void
 Client::setWorkspace(uint workspace)
 {
     if (workspace != NET_WM_STICKY_WINDOW) {
-        if (workspace >= Workspaces::instance()->size()) {
-            workspace = Workspaces::instance()->size() - 1;
+        if (workspace >= Workspaces::size()) {
+            workspace = Workspaces::size() - 1;
         }
         _workspace = workspace;
 
@@ -546,6 +549,12 @@ Client::setWorkspace(uint workspace)
 void
 Client::giveInputFocus(void)
 {
+    Frame *frame;
+    if (demandsAttention() && (frame = static_cast<Frame *>(_parent))) {
+        setDemandsAttention(false);
+        frame->removeAttention();
+    }
+
     if (_wm_hints_input) {
         PWinObj::giveInputFocus();
     }
@@ -557,12 +566,12 @@ Client::giveInputFocus(void)
 void
 Client::reparent(PWinObj *parent, int x, int y)
 {
-    XSelectInput(_dpy, _window, NoEventMask);
+    X11::selectInput(_window, NoEventMask);
     PWinObj::reparent(parent, x, y);
     _gm.x = parent->getX() + x;
     _gm.y = parent->getY() + y;
-    XSelectInput(_dpy, _window,
-                 PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
+    X11::selectInput(_window,
+                         PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
 }
 
 ActionEvent*
@@ -644,7 +653,7 @@ Client*
 Client::findClient(Window win)
 {
     // Validate input window.
-    if ((win == None) || (win == PScreen::instance()->getRoot())) {
+    if ((win == None) || (win == X11::getRoot())) {
         return 0;
     }
 
@@ -665,7 +674,7 @@ Client*
 Client::findClientFromWindow(Window win)
 {
     // Validate input window.
-    if (! win || win == PScreen::instance()->getRoot()) {
+    if (! win || win == X11::getRoot()) {
         return 0;
     }
 
@@ -751,12 +760,12 @@ Client::mapOrUnmapTransients(Window win, bool hide)
 bool
 Client::validate(void)
 {
-    XSync(_dpy, false);
+    XSync(X11::getDpy(), false);
 
     XEvent ev;
-    if (XCheckTypedWindowEvent(_dpy, _window, DestroyNotify, &ev)
-        || XCheckTypedWindowEvent(_dpy, _window, UnmapNotify, &ev)) {
-        XPutBackEvent(_dpy, &ev);
+    if (XCheckTypedWindowEvent(X11::getDpy(), _window, DestroyNotify, &ev)
+        || XCheckTypedWindowEvent(X11::getDpy(), _window, UnmapNotify, &ev)) {
+        XPutBackEvent(X11::getDpy(), &ev);
         return false;
     }
 
@@ -768,7 +777,7 @@ bool
 Client::isViewable(void)
 {
     XWindowAttributes attr;
-    XGetWindowAttributes(_dpy, _window, &attr);
+    XGetWindowAttributes(X11::getDpy(), _window, &attr);
 
     return (attr.map_state == IsViewable);
 }
@@ -778,7 +787,7 @@ void
 Client::grabButtons(void)
 {
     // Make sure we don't have any buttons grabbed.
-    XUngrabButton(_dpy, AnyButton, AnyModifier, _window);
+    X11::ungrabButton(_window);
 
     list<ActionEvent> *actions = Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
     list<ActionEvent>::iterator it(actions->begin());
@@ -791,12 +800,12 @@ Client::grabButtons(void)
 
             grabButton(it->sym, it->mod,
                        ButtonPressMask|ButtonReleaseMask,
-                       _window, None);
+                       _window);
         } else if (it->type == MOUSE_EVENT_MOTION) {
             // FIXME: Add support for MOD_ANY
             grabButton(it->sym, it->mod,
                        ButtonPressMask|ButtonReleaseMask|ButtonMotionMask,
-                       _window, None);
+                       _window);
         }
     }
 }
@@ -831,30 +840,6 @@ Client::readHints(void)
     getWMProtocols();
 }
 
-/**
- * Read WM Hints, return the initial state of the window.
- */
-ulong
-Client::readWmHints(void)
-{
-    ulong initial_state = NormalState;
-    XWMHints* hints = XGetWMHints(_dpy, _window);
-    if (hints) {
-        // get the input focus mode
-        if (hints->flags&InputHint) { // FIXME: More logic needed
-            _wm_hints_input = hints->input;
-        }
-
-        // Get initial state of the window
-        if (hints->flags&StateHint) {
-            initial_state = hints->initial_state;
-        }
-
-        XFree(hints);
-    }
-    return initial_state;
-}
-
 /**
  * Read the WM_CLASS hint and set information on the _class_hint used
  * in matching auto properties.
@@ -864,7 +849,7 @@ Client::readClassRoleHints(void)
 {
     // class hint
     XClassHint class_hint;
-    if (XGetClassHint(_dpy, _window, &class_hint)) {
+    if (XGetClassHint(X11::getDpy(), _window, &class_hint)) {
         _class_hint->h_name = Util::to_wide_str(class_hint.res_name);
         _class_hint->h_class = Util::to_wide_str(class_hint.res_class);
         XFree(class_hint.res_name);
@@ -886,48 +871,16 @@ Client::readEwmhHints(void)
     long workspace = -1;
     AtomUtil::getLong(_window, Atoms::getAtom(NET_WM_DESKTOP), workspace);
     if (workspace < 0) {
-        _workspace = Workspaces::instance()->getActive();
+        _workspace = Workspaces::getActive();
         AtomUtil::setLong(_window, Atoms::getAtom(NET_WM_DESKTOP), _workspace);
     } else {
         _workspace = workspace;
     }
 
-    // try to figure out what kind of window we are and alter it accordingly
-    int items;
-    Atom *atoms = 0;
-
-    AtomName window_type = WINDOW_TYPE;
-    atoms = (Atom*) AtomUtil::getEwmhPropData(_window, Atoms::getAtom(WINDOW_TYPE), XA_ATOM, items);
-    if (atoms) {
-      for (int i = 0; window_type == WINDOW_TYPE && i < items; ++i) {
-        if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DESKTOP)) {
-          window_type = WINDOW_TYPE_DESKTOP;
-        } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DOCK)) {
-          window_type = WINDOW_TYPE_DOCK;
-        } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_TOOLBAR)) {
-          window_type = WINDOW_TYPE_TOOLBAR;
-        } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_MENU)) {
-          window_type = WINDOW_TYPE_MENU;
-        } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_UTILITY)) {
-          window_type = WINDOW_TYPE_UTILITY;
-        } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DIALOG)) {
-          window_type = WINDOW_TYPE_DIALOG;
-        } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_SPLASH)) {
-          window_type = WINDOW_TYPE_SPLASH;
-        }
-      }
-
-      XFree(atoms);
-    }
-
-    // Set window type to WINDOW_TYPE_NORMAL if it did not match
-    if (window_type == WINDOW_TYPE) {
-        window_type = WINDOW_TYPE_NORMAL;
-        AtomUtil::setAtom(_window, Atoms::getAtom(WINDOW_TYPE), Atoms::getAtom(WINDOW_TYPE_NORMAL));
-    }
+    updateWinType(true);
 
     // Apply autoproperties for window type
-    AutoProperty *auto_property = AutoProperties::instance()->findWindowTypeProperty(window_type);
+    AutoProperty *auto_property = AutoProperties::instance()->findWindowTypeProperty(_window_type);
     if (auto_property) {
         applyAutoprops(auto_property);
     }
@@ -949,6 +902,7 @@ Client::readEwmhHints(void)
             setLayer(LAYER_BELOW);
         }
         if (win_states.fullscreen) _state.fullscreen = true;
+        _demands_attention = win_states.demands_attention;
     }
 
     // check if we have a strut
@@ -1020,10 +974,10 @@ Client::readPekwmHints(void)
 void
 Client::readIcon(void)
 {
-    PImageIcon *image = new PImageIcon(_dpy);
+    PImageIcon *image = new PImageIcon;
     if (image->loadFromWindow(_window)) {
         if (! _icon) {
-            _icon = new PTextureImage(_dpy);
+            _icon = new PTextureImage;
             TextureHandler::instance()->referenceTexture(_icon);
         }
 
@@ -1107,11 +1061,9 @@ Client::applyAutoprops(AutoProperty *ap)
     if (ap->isMask(AP_DISALLOWED_ACTIONS)) {
         applyActionAccessMask(ap->disallowed_actions, false);
     }
-#ifdef OPACITY
     if (ap->isMask(AP_OPACITY)) {
         setOpacity(ap->focus_opacity, ap->unfocus_opacity);
     }
-#endif // OPACITY
 }
 
 void
@@ -1328,7 +1280,7 @@ Client::setWmState(ulong state)
     data[0] = state;
     data[1] = None; // No Icon
 
-    XChangeProperty(_dpy, _window,
+    XChangeProperty(X11::getDpy(), _window,
                     Atoms::getAtom(WM_STATE),
                     Atoms::getAtom(WM_STATE),
                     32, PropModeReplace, (uchar*) data, 2);
@@ -1349,7 +1301,7 @@ Client::getWmState(void)
     uchar *udata;
 
     int status =
-        XGetWindowProperty(_dpy, _window, Atoms::getAtom(WM_STATE),
+        XGetWindowProperty(X11::getDpy(), _window, Atoms::getAtom(WM_STATE),
                            0L, 2L, False, Atoms::getAtom(WM_STATE),
                            &real_type, &real_format, &items_read, &items_left,
                            &udata);
@@ -1383,7 +1335,7 @@ Client::configureRequestSend(void)
     e.above = None;
     e.override_redirect = False;
 
-    XSendEvent(_dpy, _window, false, StructureNotifyMask, (XEvent *) &e);
+    XSendEvent(X11::getDpy(), _window, false, StructureNotifyMask, (XEvent *) &e);
 }
 
 //! @brief Send a TAKE_FOCUS client message to the client (if requested by it).
@@ -1392,14 +1344,18 @@ void Client::sendTakeFocusMessage(void)
     if (_send_focus_message) {
         {
             XEvent ev;
-            XChangeProperty(_dpy, RootWindow(_dpy, DefaultScreen(_dpy)), XA_PRIMARY, XA_STRING, 8, PropModeAppend, 0, 0);
-            XWindowEvent(_dpy, RootWindow(_dpy, DefaultScreen(_dpy)), PropertyChangeMask, &ev);
-            PScreen::instance()->setLastEventTime(ev.xproperty.time);
+            XChangeProperty(X11::getDpy(),
+                            RootWindow(X11::getDpy(), DefaultScreen(X11::getDpy())),
+                            XA_PRIMARY, XA_STRING, 8, PropModeAppend, 0, 0);
+            XWindowEvent(X11::getDpy(),
+                         RootWindow(X11::getDpy(), DefaultScreen(X11::getDpy())),
+                         PropertyChangeMask, &ev);
+            X11::setLastEventTime(ev.xproperty.time);
         }
-        sendXMessage(_window,
-                     Atoms::getAtom(WM_PROTOCOLS), NoEventMask,
-                     Atoms::getAtom(WM_TAKE_FOCUS),
-                     PScreen::instance()->getLastEventTime());
+        X11::sendEvent(_window,
+                       Atoms::getAtom(WM_PROTOCOLS), NoEventMask,
+                       Atoms::getAtom(WM_TAKE_FOCUS),
+                       X11::getLastEventTime());
     }
 }
 
@@ -1407,33 +1363,25 @@ void Client::sendTakeFocusMessage(void)
 //! Grabs the button button, with the mod mod and mask mask on the window win
 //! and cursor curs with "all" possible extra modifiers
 void
-Client::grabButton(int button, int mod, int mask, Window win, Cursor curs)
+Client::grabButton(int button, int mod, int mask, Window win)
 {
-    uint num_lock = PScreen::instance()->getNumLock();
-    uint scroll_lock = PScreen::instance()->getScrollLock();
+    uint num_lock = X11::getNumLock();
+    uint scroll_lock = X11::getScrollLock();
 
-    XGrabButton(_dpy, button, mod,
-                win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
-    XGrabButton(_dpy, button, mod|LockMask,
-                win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
+    X11::grabButton(button, mod, win, mask);
+    X11::grabButton(button, mod|LockMask, win, mask);
 
     if (num_lock) {
-        XGrabButton(_dpy, button, mod|num_lock,
-                    win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
-        XGrabButton(_dpy, button, mod|num_lock|LockMask,
-                    win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
+        X11::grabButton(button, mod|num_lock, win, mask);
+        X11::grabButton(button, mod|num_lock|LockMask, win, mask);
     }
     if (scroll_lock) {
-        XGrabButton(_dpy, button, mod|scroll_lock,
-                    win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
-        XGrabButton(_dpy, button, mod|scroll_lock|LockMask,
-                    win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
+        X11::grabButton(button, mod|scroll_lock, win, mask);
+        X11::grabButton(button, mod|scroll_lock|LockMask, win, mask);
     }
     if (num_lock && scroll_lock) {
-        XGrabButton(_dpy, button, mod|num_lock|scroll_lock,
-                    win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
-        XGrabButton(_dpy, button, mod|num_lock|scroll_lock|LockMask,
-                    win, true, mask, GrabModeAsync, GrabModeAsync, None, curs);
+        X11::grabButton(button, mod|num_lock|scroll_lock, win, mask);
+        X11::grabButton(button, mod|num_lock|scroll_lock|LockMask, win, mask);
     }
 }
 
@@ -1465,22 +1413,12 @@ Client::setSkip(uint skip)
     AtomUtil::setLong(_window, Atoms::getAtom(PEKWM_FRAME_SKIP), _state.skip);
 }
 
-/**
- * Set demands attention state, this should be unset when client
- * recieves focus. This is ignored if client has focus.
- */
-void
-Client::setStateDemandsAttention(StateAction sa, bool attention)
-{
-    // FIXME: Demands attention state.
-}
-
 //! @brief Sends an WM_DELETE message to the client, else kills it.
 void
 Client::close(void)
 {
     if (_send_close_message) {
-        sendXMessage(_window, Atoms::getAtom(WM_PROTOCOLS), NoEventMask,
+        X11::sendEvent(_window, Atoms::getAtom(WM_PROTOCOLS), NoEventMask,
                      Atoms::getAtom(WM_DELETE_WINDOW), CurrentTime);
     } else {
         kill();
@@ -1491,7 +1429,7 @@ Client::close(void)
 void
 Client::kill(void)
 {
-    XKillClient(_dpy, _window);
+    XKillClient(X11::getDpy(), _window);
 }
 
 //! @brief Sets the position based on P or U position.
@@ -1630,7 +1568,7 @@ Client::getMwmHints(Window win)
 
     Atom hints_atom = Atoms::getAtom(MOTIF_WM_HINTS);
 
-    int status = XGetWindowProperty(_dpy, win, hints_atom, 0L, 20L, False, hints_atom,
+    int status = XGetWindowProperty(X11::getDpy(), win, hints_atom, 0L, 20L, False, hints_atom,
                                     &real_type, &real_format, &items_read, &items_left, &udata);
 
     if (status == Success) {
@@ -1664,31 +1602,10 @@ Client::handleColormapChange(XColormapEvent *e)
 {
     if (e->c_new) {
         _cmap = e->colormap;
-        XInstallColormap(_dpy, _cmap);
+        XInstallColormap(X11::getDpy(), _cmap);
     }
 }
 
-int
-Client::sendXMessage(Window window, Atom atom, long mask,
-                     long v1, long v2, long v3, long v4, long v5)
-{
-    XEvent e;
-
-    e.type = e.xclient.type = ClientMessage;
-    e.xclient.display = _dpy;
-    e.xclient.window = window;
-    e.xclient.format = 32;
-    e.xclient.message_type = atom;
-
-    e.xclient.data.l[0] = v1;
-    e.xclient.data.l[1] = v2;
-    e.xclient.data.l[2] = v3;
-    e.xclient.data.l[3] = v4;
-    e.xclient.data.l[4] = v5;
-
-    return XSendEvent(_dpy, window, false, mask, &e);
-}
-
 //! @brief
 bool
 Client::getEwmhStates(NetWMStates &win_states)
@@ -1781,11 +1698,76 @@ Client::updateEwmhStates(void)
     delete [] atoms;
 }
 
+void
+Client::updateWinType(bool set)
+{
+    // try to figure out what kind of window we are and alter it accordingly
+    int items;
+    Atom *atoms = 0;
+
+    _window_type = WINDOW_TYPE;
+    atoms = (Atom*) AtomUtil::getEwmhPropData(_window, Atoms::getAtom(WINDOW_TYPE), XA_ATOM, items);
+    if (atoms) {
+        for (int i = 0; _window_type == WINDOW_TYPE && i < items; ++i) {
+            if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DESKTOP)) {
+                _window_type = WINDOW_TYPE_DESKTOP;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DOCK)) {
+                _window_type = WINDOW_TYPE_DOCK;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_TOOLBAR)) {
+                _window_type = WINDOW_TYPE_TOOLBAR;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_MENU)) {
+                _window_type = WINDOW_TYPE_MENU;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_UTILITY)) {
+                _window_type = WINDOW_TYPE_UTILITY;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DIALOG)) {
+                _window_type = WINDOW_TYPE_DIALOG;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_SPLASH)) {
+                _window_type = WINDOW_TYPE_SPLASH;
+            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_NORMAL)) {
+                _window_type = WINDOW_TYPE_NORMAL;
+            }
+        }
+        XFree(atoms);
+    }
+
+    // Set window type to WINDOW_TYPE_NORMAL if it did not match
+    if (_window_type == WINDOW_TYPE) {
+        _window_type = WINDOW_TYPE_NORMAL;
+        if (set)
+            AtomUtil::setAtom(_window, Atoms::getAtom(WINDOW_TYPE), Atoms::getAtom(WINDOW_TYPE_NORMAL));
+    }
+}
+
+/**
+ * Read WM Hints, return the initial state of the window.
+ */
+ulong
+Client::getWMHints(void)
+{
+    ulong initial_state = NormalState;
+    XWMHints* hints = XGetWMHints(X11::getDpy(), _window);
+    if (hints) {
+        // get the input focus mode
+        if (hints->flags&InputHint) { // FIXME: More logic needed
+            _wm_hints_input = hints->input;
+        }
+
+        // Get initial state of the window
+        if (hints->flags&StateHint) {
+            initial_state = hints->initial_state;
+        }
+
+        setDemandsAttention(hints->flags&XUrgencyHint);
+        XFree(hints);
+    }
+    return initial_state;
+}
+
 void
 Client::getWMNormalHints(void)
 {
     long dummy;
-    XGetWMNormalHints(_dpy, _window, _size, &dummy);
+    XGetWMNormalHints(X11::getDpy(), _window, _size, &dummy);
 
     // let's do some sanity checking
     if (_size->flags & PBaseSize) {
@@ -1830,7 +1812,7 @@ Client::getWMProtocols(void)
     int count;
     Atom *protocols;
 
-    if (XGetWMProtocols(_dpy, _window, &protocols, &count) != 0) {
+    if (XGetWMProtocols(X11::getDpy(), _window, &protocols, &count) != 0) {
         for (int i = 0; i < count; ++i) {
             if (protocols[i] == Atoms::getAtom(WM_TAKE_FOCUS)) {
                 _send_focus_message = true;
@@ -1850,7 +1832,7 @@ void
 Client::getTransientForHint(void)
 {
     if (! _transient) {
-        XGetTransientForHint(_dpy, _window, &_transient_window);
+        XGetTransientForHint(X11::getDpy(), _window, &_transient_window);
 
         if (_transient_window != None) {
             _transient = findClientFromWindow(_transient_window);
@@ -1872,18 +1854,18 @@ Client::getStrutHint(void)
                                                                XA_CARDINAL, num));
     if (strut) {
         if (_strut) {
-            PScreen::instance()->removeStrut(_strut);
+            X11::removeStrut(_strut);
         } else {
             _strut = new Strut();
         }
 
         *_strut = strut;
-        PScreen::instance()->addStrut(_strut);
+        X11::addStrut(_strut);
 
         XFree(strut);
 
     } else if (_strut) {
-        PScreen::instance()->removeStrut(_strut);
+        X11::removeStrut(_strut);
         delete _strut;
         _strut = 0;
     }
@@ -1896,7 +1878,7 @@ Client::removeStrutHint(void)
     if (! _strut)
         return;
 
-    PScreen::instance()->removeStrut(_strut);
+    X11::removeStrut(_strut);
     delete _strut;
     _strut = 0;
 }
