@@ -14,7 +14,7 @@
 
 #include "Compat.hh"
 #include "Util.hh"
-#include "PScreen.hh" // for DPY in keyconfig code
+#include "x11.hh" // for DPY in keyconfig code
 
 #include <iostream>
 #include <fstream>
@@ -93,9 +93,7 @@ Config::Config(void) :
         _screen_show_status_window(true), _screen_show_status_window_on_root(false),
         _screen_show_client_id(false),
         _screen_show_workspace_indicator(500), _screen_workspace_indicator_scale(16),
-#ifdef OPACITY
         _screen_workspace_indicator_opacity(EWMH_OPAQUE_WINDOW),
-#endif // OPACITY
         _screen_place_new(true), _screen_focus_new(false),
         _screen_focus_new_child(true), _screen_honour_randr(true),
         _screen_honour_aspectratio(true),
@@ -107,18 +105,14 @@ Config::Config(void) :
 	_screen_report_all_clients(false),
         _menu_select_mask(0), _menu_enter_mask(0), _menu_exec_mask(0),
         _menu_display_icons(true),
-#ifdef OPACITY
         _menu_focus_opacity(EWMH_OPAQUE_WINDOW),
         _menu_unfocus_opacity(EWMH_OPAQUE_WINDOW),
-#endif // OPACITY
         _cmd_dialog_history_unique(true), _cmd_dialog_history_size(1024),
         _cmd_dialog_history_file("~/.pekwm/history"), _cmd_dialog_history_save_interval(16),
         _harbour_da_min_s(0), _harbour_da_max_s(0),
         _harbour_ontop(true), _harbour_maximize_over(false),
-        _harbour_placement(TOP), _harbour_orientation(TOP_TO_BOTTOM), _harbour_head_nr(0)
-#ifdef OPACITY
-       ,_harbour_opacity(EWMH_OPAQUE_WINDOW)
-#endif // OPACITY
+        _harbour_placement(TOP), _harbour_orientation(TOP_TO_BOTTOM), _harbour_head_nr(0),
+        _harbour_opacity(EWMH_OPAQUE_WINDOW)
 {
     if (_instance) {
         throw string("Config, trying to create multiple instances");
@@ -181,9 +175,7 @@ Config::Config(void) :
     _action_map["SubMenu"] = pair<ActionType, uint>(ACTION_MENU_SUB, ROOTMENU_OK|WINDOWMENU_OK);
     _action_map["Dynamic"] = pair<ActionType, uint>(ACTION_MENU_DYN, ROOTMENU_OK|WINDOWMENU_OK);
     _action_map["SendKey"] = pair<ActionType, uint>(ACTION_SEND_KEY, ANY_MASK);
-#ifdef OPACITY
     _action_map["SetOpacity"] = pair<ActionType, uint>(ACTION_SET_OPACITY, FRAME_MASK);
-#endif // OPACITY
 
     _action_access_mask_map[""] = ACTION_ACCESS_NO;
     _action_access_mask_map["MOVE"] = ACTION_ACCESS_MOVE;
@@ -327,9 +319,7 @@ Config::Config(void) :
     _action_state_map["Marked"] = ACTION_STATE_MARKED;
     _action_state_map["Skip"] = ACTION_STATE_SKIP;
     _action_state_map["CfgDeny"] = ACTION_STATE_CFG_DENY;
-#ifdef OPACITY
     _action_state_map["Opaque"] = ACTION_STATE_OPAQUE;
-#endif // OPACITY
     _action_state_map["Title"] = ACTION_STATE_TITLE;
     _action_state_map["HarbourHidden"] = ACTION_STATE_HARBOUR_HIDDEN;
     _action_state_map["GlobalGrouping"] = ACTION_STATE_GLOBAL_GROUPING;
@@ -604,10 +594,8 @@ Config::loadScreen(CfgParser::Entry *section)
                                            _screen_show_workspace_indicator, 500, 0));
     key_list.push_back(new CfgParserKeyNumeric<int>("WORKSPACEINDICATORSCALE",
                                            _screen_workspace_indicator_scale, 16, 2)); 
-#ifdef OPACITY
     key_list.push_back(new CfgParserKeyNumeric<uint>("WORKSPACEINDICATOROPACITY",
                                            _screen_workspace_indicator_opacity, 100, 0, 100));
-#endif // OPACITY
     key_list.push_back(new CfgParserKeyBool("PLACENEW", _screen_place_new));
     key_list.push_back(new CfgParserKeyBool("FOCUSNEW", _screen_focus_new));
     key_list.push_back(new CfgParserKeyBool("FOCUSNEWCHILD", _screen_focus_new_child, true));
@@ -725,10 +713,8 @@ Config::loadMenu(CfgParser::Entry *section)
     key_list.push_back(new CfgParserKeyString("ENTER", value_enter, "BUTTONPRESS", 0));
     key_list.push_back(new CfgParserKeyString("EXEC", value_exec, "BUTTONRELEASE", 0));
     key_list.push_back(new CfgParserKeyBool("DISPLAYICONS", _menu_display_icons, true));
-#ifdef OPACITY
     key_list.push_back(new CfgParserKeyNumeric<uint>("FOCUSOPACITY", _menu_focus_opacity, 100, 0, 100));
     key_list.push_back(new CfgParserKeyNumeric<uint>("UNFOCUSOPACITY", _menu_unfocus_opacity, 100, 0, 100));
-#endif // OPACITY
 
     // Parse data
     section->parse_key_values(key_list.begin(), key_list.end());
@@ -819,9 +805,7 @@ Config::loadHarbour(CfgParser::Entry *section)
     key_list.push_back(new CfgParserKeyNumeric<int>("HEAD", _harbour_head_nr, 0, 0));
     key_list.push_back(new CfgParserKeyString("PLACEMENT", value_placement, "RIGHT", 0));
     key_list.push_back(new CfgParserKeyString("ORIENTATION", value_orientation, "TOPTOBOTTOM", 0));
-#ifdef OPACITY
     key_list.push_back(new CfgParserKeyNumeric<uint>("OPACITY", _harbour_opacity, 100, 0, 100));
-#endif // OPACITY
 
     // Parse data
     section->parse_key_values(key_list.begin(), key_list.end());
@@ -918,7 +902,7 @@ Config::parseKey(const std::string &key_string, uint &mod, uint &key)
                     }
                 }
             }
-            key = XKeysymToKeycode(PScreen::instance()->getDpy(), keysym);
+            key = XKeysymToKeycode(X11::getDpy(), keysym);
         }
 
         // if the last token isn't an key/button, the action isn't valid
@@ -1082,7 +1066,6 @@ Config::parseAction(const std::string &action_string, Action &action, uint mask)
                         action.setParamI(0, false); // Default to non-sticky
                     }
                     break;
-#ifdef OPACITY
                 case ACTION_SET_OPACITY:
                     if ((Util::splitString(tok[1], tok, " \t", 2)) == 2) {
                         action.setParamI(0, std::atoi(tok[tok.size() - 2].c_str()));
@@ -1092,7 +1075,6 @@ Config::parseAction(const std::string &action_string, Action &action, uint mask)
                         action.setParamI(1, std::atoi(tok[1].c_str()));
                     }
                     break;
-#endif // OPACITY
                 default:
                     // do nothing
                     break;
@@ -1858,7 +1840,6 @@ Config::parseWorkspaceNumber(const std::string &workspace)
     return num;
 }
 
-#ifdef OPACITY
 //! @brief Parses a string which contains two opacity values
 bool
 Config::parseOpacity(const std::string value, uint &focused, uint &unfocused)
@@ -1879,4 +1860,4 @@ Config::parseOpacity(const std::string value, uint &focused, uint &unfocused)
     CONV_OPACITY(unfocused);
     return true;
 }
-#endif // OPACITY
+
