@@ -1,6 +1,6 @@
 //
 // FontHandler.cc for pekwm
-// Copyright © 2004-2008 Claes Nästén <me@pekdon.net>
+// Copyright © 2004-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -13,7 +13,6 @@
 #include <cctype>
 #include <iostream>
 
-#include "PScreen.hh"
 #include "ColorHandler.hh"
 #include "FontHandler.hh"
 #include "Util.hh"
@@ -28,14 +27,13 @@ using std::string;
 FontHandler* FontHandler::_instance = 0;
 
 //! @brief FontHandler constructor
-FontHandler::FontHandler(void) :
-  _free_on_return_font(false), _free_on_return_color(false)
+FontHandler::FontHandler(void)
 {
 #ifdef DEBUG
     if (_instance) {
         cerr << __FILE__ << "@" << __LINE__ << ": "
             << "FontHandler(" << this << ")::FontHandler()"
-            << endl << " *** _instance allready set" << endl;
+            << endl << " *** _instance already set" << endl;
     }
 #endif // DEBUG
   
@@ -93,7 +91,7 @@ FontHandler::getFont(const std::string &font)
 
     vector<string> tok;
     vector<string>::iterator tok_it; // old gcc doesn't like --tok.end()
-    if ((Util::splitString(font, tok, "#")) > 1) {
+    if ((Util::splitString(font, tok, "#", 0, true)) > 1) {
         // Try getting the font type from the first paramter, if that
         // doesn't work fall back to the last. This is to backwards
         // compatible.
@@ -106,21 +104,21 @@ FontHandler::getFont(const std::string &font)
 
         switch (type) {
         case PFont::FONT_TYPE_XMB:
-            pfont = new PFontXmb(PScreen::instance());
+            pfont = new PFontXmb;
             tok.erase(tok_it);
             break;
 #ifdef HAVE_XFT
         case PFont::FONT_TYPE_XFT:
-            pfont = new PFontXft(PScreen::instance());
+            pfont = new PFontXft;
             tok.erase(tok_it);
             break;
 #endif // HAVE_XFT
         case PFont::FONT_TYPE_X11:
-            pfont = new PFontX11(PScreen::instance());
+            pfont = new PFontX11;
             tok.erase(tok_it);
             break;
         default:
-            pfont = new PFontXmb(PScreen::instance());
+            pfont = new PFontXmb;
             break;
         };
         pfont->load(tok.front());
@@ -146,7 +144,7 @@ FontHandler::getFont(const std::string &font)
             }
         }
     } else {
-        pfont = new PFontXmb(PScreen::instance());
+        pfont = new PFontXmb;
         pfont->load(font);
     }
 
@@ -168,7 +166,7 @@ FontHandler::returnFont(PFont *font)
     for (; it != _font_list.begin(); ++it) {
         if (it->getData() == font) {
             it->decRef();
-            if ((it->getRef() == 0) && _free_on_return_font) {
+            if (! it->getRef()) {
                 delete it->getData();
                 _font_list.erase(it);
             }
@@ -183,7 +181,7 @@ FontHandler::getColor(const std::string &color)
 {
     // check cache
     list<HandlerEntry<PFont::Color*> >::iterator it(_color_list.begin());
-    for (; it != _color_list.begin(); ++it) {
+    for (; it != _color_list.end(); ++it) {
         if (*it == color) {
             it->incRef();
             return it->getData();
@@ -221,7 +219,7 @@ FontHandler::returnColor(PFont::Color *color)
     for (; it != _color_list.begin(); ++it) {
         if (it->getData() == color) {
             it->decRef();
-            if ((it->getRef() == 0) && _free_on_return_color) {
+            if (! it->getRef()) {
                 delete it->getData();
                 _color_list.erase(it);
             }
@@ -230,36 +228,6 @@ FontHandler::returnColor(PFont::Color *color)
     }
 }
 
-//! @brief Frees all fonts not beeing in use
-void
-FontHandler::freeUnrefFonts(void)
-{
-    list<HandlerEntry<PFont*> >::iterator it(_font_list.begin());
-    while (it != _font_list.end()) {
-        if (it->getRef() == 0) {
-            delete it->getData();
-            it = _font_list.erase(it);
-        } else {
-            ++it;
-        }
-    }
-}
-
-//! @brief Frees all colors not beeing in use
-void
-FontHandler::freeUnrefColors(void)
-{
-    list<HandlerEntry<PFont::Color*> >::iterator it(_color_list.begin());
-    while (it != _color_list.end()) {
-        if (it->getRef() == 0) {
-            freeColor(it->getData());
-            it = _color_list.erase(it);
-        } else {
-            ++it;
-        }
-    }
-}
-
 //! @brief Helper loader of font colors ( main and offset color )
 void
 FontHandler::loadColor(const std::string &color, PFont::Color *font_color, bool fg)
@@ -267,7 +235,7 @@ FontHandler::loadColor(const std::string &color, PFont::Color *font_color, bool
     XColor *xc;
 
     vector<string> tok;
-    if (Util::splitString(color, tok, ",", 2) == 2) {
+    if (Util::splitString(color, tok, ",", 2, true) == 2) {
         uint alpha = static_cast<uint>(strtol(tok[1].c_str(), 0, 10));
         if (alpha > 100) {
             cerr << " *** WARNING: Alpha for font color greater than 100%" << endl;
