@@ -25,7 +25,7 @@ extern "C" {
 #include "Frame.hh"
 
 #include "Compat.hh"
-#include "PScreen.hh"
+#include "x11.hh"
 #include "Config.hh"
 #include "ActionHandler.hh"
 #include "AutoProperties.hh"
@@ -59,11 +59,9 @@ Frame::Frame(Client *client, AutoProperty *ap)
     : PDecor(WindowManager::instance()->getTheme(),
              Frame::getClientDecorName(client), client->getWindow()),
       _id(0), _client(0), _class_hint(0),
-      _non_fullscreen_decor_state(0), _non_fullscreen_layer(LAYER_NORMAL)
+      _non_fullscreen_decor_state(0), _non_fullscreen_layer(LAYER_NORMAL),
+      _attention(0)
 {
-    // setup basic pointers
-    _scr = WindowManager::instance()->getScreen();
-
     // PWinObj attributes
     _type = WO_FRAME;
 
@@ -75,9 +73,8 @@ Frame::Frame(Client *client, AutoProperty *ap)
 
     // grab buttons so that we can reply them
     for (uint i = 0; i < BUTTON_NO; ++i) {
-        XGrabButton(_dpy, i, AnyModifier, _window,
-                    True, ButtonPressMask|ButtonReleaseMask,
-                    GrabModeSync, GrabModeAsync, None, None);
+        X11::grabButton(i, AnyModifier, _window,
+                        ButtonPressMask|ButtonReleaseMask, GrabModeSync);
     }
 
     // get unique id of the frame, if the client didn't have an id
@@ -95,7 +92,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
     _class_hint = new ClassHint();
     *_class_hint = *client->getClassHint();
 
-    _scr->grabServer();
+    X11::grabServer();
 
     // We don't send any ConfigurRequests during setup, we send one when we
     // are finished to minimize traffic and confusion
@@ -146,14 +143,14 @@ Frame::Frame(Client *client, AutoProperty *ap)
 
     // still need a position?
     if (place) {
-        Workspaces::instance()->placeWo(this, client->getTransientClientWindow());
+        Workspaces::placeWo(this, client->getTransientClientWindow());
     }
 
     _old_gm = _gm;
     _non_fullscreen_decor_state = client->getDecorState();
     _non_fullscreen_layer = client->getLayer();
 
-    _scr->ungrabServer(true); // ungrab and sync
+    X11::ungrabServer(true); // ungrab and sync
 
     // now insert the client in the frame we created.
     addChild(client);
@@ -166,7 +163,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
     // be able to skip an extra updateClientList
     _frame_list.push_back(this);
     WindowManager::instance()->addToFrameList(this);
-    Workspaces::instance()->insert(this);
+    Workspaces::insert(this);
 
     activateChild(client);
 
@@ -196,7 +193,7 @@ Frame::~Frame(void)
     _wo_map.erase(_window);
     woListRemove(this);
     _frame_list.remove(this);
-    Workspaces::instance()->remove(this);
+    Workspaces::remove(this);
     WindowManager::instance()->removeFromFrameList(this);
     if (_tag_frame == this) {
         _tag_frame = 0;
@@ -208,8 +205,8 @@ Frame::~Frame(void)
         delete _class_hint;
     }
 
-    Workspaces::instance()->updateClientList();
-    Workspaces::instance()->updateClientStackingList();
+    Workspaces::updateClientList();
+    Workspaces::updateClientStackingList();
 }
 
 // START - PWinObj interface.
@@ -236,7 +233,7 @@ Frame::stick(void)
     _sticky = ! _sticky;
 
     // make sure it's visible/hidden
-    PDecor::setWorkspace(Workspaces::instance()->getActive());
+    PDecor::setWorkspace(Workspaces::getActive());
 }
 
 void
@@ -297,7 +294,7 @@ Frame::handleMotionEvent(XMotionEvent *ev)
     }
 
     ActionEvent *ae = 0;
-    uint button = _scr->getButtonFromState(ev->state);
+    uint button = X11::getButtonFromState(ev->state);
 
     if (ev->window == getTitleWindow()) {
         ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION,
@@ -389,7 +386,7 @@ Frame::handleMapRequest(XMapRequestEvent *ev)
         return 0;
     }
 
-    if (! _sticky && (_workspace != Workspaces::instance()->getActive())) {
+    if (! _sticky && (_workspace != Workspaces::getActive())) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "Ignoring MapRequest, not on current workspace!" << endl;
@@ -452,11 +449,15 @@ Frame::getActiveClient(void)
 
 //! @brief Adds child to the frame.
 void
-Frame::addChild (PWinObj *child, std::list<PWinObj*>::iterator *it)
+Frame::addChild(PWinObj *child, std::list<PWinObj*>::iterator *it)
 {
     PDecor::addChild(child, it);
     AtomUtil::setLong(child->getWindow(), Atoms::getAtom(PEKWM_FRAME_ID), _id);
     child->lower();
+
+    Client *client = dynamic_cast<Client*>(child);
+    if (client && client->demandsAttention())
+        ++_attention;
 }
 
 /**
@@ -490,7 +491,7 @@ Frame::removeChild (PWinObj *child, bool do_delete)
  * rules to match updated title.
  */
 void
-Frame::activateChild (PWinObj *child)
+Frame::activateChild(PWinObj *child)
 {
     // FIXME: Update default decoration for this child, can change
     // decoration from DEFAULT to REMOTE or WARNING
@@ -501,17 +502,19 @@ Frame::activateChild (PWinObj *child)
     }
 
     _client = static_cast<Client*>(child);
+    if (_client->demandsAttention()) {
+        _client->setDemandsAttention(false);
+        --_attention;
+    }
     PDecor::activateChild(child);
 
     // setShape uses current active child, so we need to activate the
     // child before setting shape
-    if (PScreen::instance()->hasExtensionShape()) {
+    if (X11::hasExtensionShape()) {
         _client->setShaped(setShape());
     }
 
-#ifdef OPACITY
     setOpacity(_client);
-#endif // OPACITY
 
     if (_focused) {
         child->giveInputFocus();
@@ -520,8 +523,8 @@ Frame::activateChild (PWinObj *child)
     // Reload decor rules if needed.
     handleTitleChange(_client);
 
-    Workspaces::instance()->updateClientList();
-    Workspaces::instance()->updateClientStackingList();
+    Workspaces::updateClientList();
+    Workspaces::updateClientStackingList();
 }
 
 /**
@@ -577,6 +580,18 @@ Frame::getDecorInfo(wchar_t *buf, uint size)
     swprintf(buf, size, L"%d+%d+%d+%d", width, height, _gm.x, _gm.y);
 }
 
+void
+Frame::giveInputFocus(void)
+{
+    if (_client->demandsAttention()) {
+        _client->setDemandsAttention(false);
+        if (! --_attention) {
+            setDecor(getClientDecorName(_client));
+        }
+    }
+    PDecor::giveInputFocus();
+}
+
 //! @brief
 void
 Frame::setShaded(StateAction sa)
@@ -778,7 +793,7 @@ Frame*
 Frame::findFrameFromWindow(Window win)
 {
     // Validate input window.
-    if ((win == None) || (win == PScreen::instance()->getRoot())) {
+    if ((win == None) || (win == X11::getRoot())) {
         return 0;
     }
 
@@ -850,7 +865,7 @@ Frame::applyAPGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
     // < 1 consider it to be full screen size.
     if (mask&WidthValue) {
         if (ap_gm.width < 1) {
-            gm.width = _scr->getWidth();
+            gm.width = X11::getWidth();
         } else {
             gm.width = ap_gm.width;
         }
@@ -867,13 +882,13 @@ Frame::applyAPGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
     if (mask&XValue) {
         gm.x = ap_gm.x;
         if (mask&XNegative) {
-            gm.x += _scr->getWidth() - gm.width;
+            gm.x += X11::getWidth() - gm.width;
         }
     }
     if (mask&YValue) {
         gm.y = ap_gm.y;
         if (mask&YNegative) {
-            gm.y += _scr->getHeight() - gm.height;
+            gm.y += X11::getHeight() - gm.height;
         }
     }
 }
@@ -915,8 +930,25 @@ Frame::returnFrameID(uint id)
 std::string
 Frame::getClientDecorName(Client *client)
 {
-  DecorProperty *prop = WindowManager::instance()->getAutoProperties()->findDecorProperty(client->getClassHint());
-  return prop ? prop->getName() : PDecor::DEFAULT_DECOR_NAME;
+    DecorProperty *dp;
+    AutoProperty *ap;
+
+    ap = WindowManager::instance()->getAutoProperties()->findAutoProperty(client->getClassHint());
+    if (ap && ap->isMask(AP_DECOR)) {
+        return ap->frame_decor;
+    }
+
+    ap = WindowManager::instance()->getAutoProperties()->findWindowTypeProperty(client->getWinType());
+    if (ap && ap->isMask(AP_DECOR)) {
+        return ap->frame_decor;
+    }
+
+    dp = WindowManager::instance()->getAutoProperties()->findDecorProperty(client->getClassHint());
+    if (dp) {
+        return dp->getName();
+    }
+
+    return PDecor::DEFAULT_DECOR_NAME;
 }
 
 //! @brief Resets Frame IDs.
@@ -945,7 +977,7 @@ Frame::detachClient(Client *client)
         Frame *frame = new Frame(client, 0);
 
         client->setParent(frame);
-        client->setWorkspace(Workspaces::instance()->getActive());
+        client->setWorkspace(Workspaces::getActive());
 
         setFocused(false);
     }
@@ -956,7 +988,7 @@ bool
 Frame::fixGeometry(void)
 {
     Geometry head, before;
-    PScreen::instance()->getHeadInfoWithEdge(getNearestHead(), head);
+    X11::getHeadInfoWithEdge(getNearestHead(), head);
 
     before = _gm;
 
@@ -1001,8 +1033,9 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
         name += L"No Name";
     }
 
-    bool status = _scr->grabPointer(_scr->getRoot(),
-                                    ButtonReleaseMask|PointerMotionMask, None);
+    bool status = X11::grabPointer(X11::getRoot(),
+                                       ButtonReleaseMask|PointerMotionMask,
+                                       None);
     if (status != true) {
         return;
     }
@@ -1016,7 +1049,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
     XEvent e;
     while (true) { // this breaks when we get an button release
-        XMaskEvent(_dpy, PointerMotionMask|ButtonReleaseMask, &e);
+        XMaskEvent(X11::getDpy(), PointerMotionMask|ButtonReleaseMask, &e);
 
         switch (e.type)  {
         case MotionNotify:
@@ -1030,7 +1063,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
         case ButtonRelease:
             sw->unmapWindow();
-            _scr->ungrabPointer();
+            X11::ungrabPointer();
 
             Client *search = 0;
 
@@ -1040,7 +1073,8 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
                 Window win;
 
                 // find the frame we dropped the client on
-                XTranslateCoordinates(_dpy, _scr->getRoot(), _scr->getRoot(),
+                XTranslateCoordinates(X11::getDpy(),
+                                      X11::getRoot(), X11::getRoot(),
                                       e.xmotion.x_root, e.xmotion.y_root,
                                       &x, &y, &win);
 
@@ -1081,7 +1115,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
                 client->setParent(frame);
 
                 // make sure the client ends up on the current workspace
-                client->setWorkspace(Workspaces::instance()->getActive());
+                client->setWorkspace(Workspaces::getActive());
 
                 // make sure it get's focus
                 setFocused(false);
@@ -1163,8 +1197,8 @@ Frame::doResize(bool left, bool x, bool top, bool y)
         return;
     }
 
-    if (! _scr->grabPointer(_scr->getRoot(), ButtonMotionMask|ButtonReleaseMask,
-                           ScreenResources::instance()->getCursor(ScreenResources::CURSOR_RESIZE))) {
+    if (! X11::grabPointer(X11::getRoot(), ButtonMotionMask|ButtonReleaseMask,
+                            ScreenResources::instance()->getCursor(ScreenResources::CURSOR_RESIZE))) {
         return;
     }
 
@@ -1174,20 +1208,20 @@ Frame::doResize(bool left, bool x, bool top, bool y)
     // Initialize variables
     int start_x, new_x;
     int start_y, new_y;
-    uint last_width, old_width;
-    uint last_height, old_height;
+    uint old_width;
+    uint old_height;
 
     start_x = new_x = left ? _gm.x : (_gm.x + _gm.width);
     start_y = new_y = top ? _gm.y : (_gm.y + _gm.height);
-    last_width = old_width = _gm.width;
-    last_height = old_height = _gm.height;
+    old_width = _gm.width;
+    old_height = _gm.height;
 
     // the basepoint of our window
     _click_x = left ? (_gm.x + _gm.width) : _gm.x;
     _click_y = top ? (_gm.y + _gm.height) : _gm.y;
 
     int pointer_x = _gm.x, pointer_y = _gm.y;
-    _scr->getMousePosition(pointer_x, pointer_y);
+    X11::getMousePosition(pointer_x, pointer_y);
 
     wchar_t buf[128];
     getDecorInfo(buf, 128);
@@ -1204,7 +1238,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
 
     // grab server, we don't want invert traces
     if (outline) {
-        _scr->grabServer();
+        X11::grabServer();
     }
 
     const long resize_mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
@@ -1214,7 +1248,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
         if (outline) {
             drawOutline(_gm);
         }
-        XMaskEvent(_dpy, resize_mask, &ev);
+        XMaskEvent(X11::getDpy(), resize_mask, &ev);
         if (outline) {
             drawOutline(_gm); // clear
         }
@@ -1222,7 +1256,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
         switch (ev.type) {
         case MotionNotify:
             // Flush all pointer motion, no need to redraw and redraw.
-            PScreen::removeMotionEvents();
+            X11::removeMotionEvents();
 
             if (x) {
                 new_x = start_x - pointer_x + ev.xmotion.x;
@@ -1249,7 +1283,6 @@ Frame::doResize(bool left, bool x, bool top, bool y)
             break;
         case ButtonRelease:
             exit = true;
-            XPutBackEvent(_dpy, &ev);
             break;
         }
     }
@@ -1258,7 +1291,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
         sw->unmapWindow();
     }
 
-    _scr->ungrabPointer();
+    X11::ungrabPointer();
 
     // Make sure the state isn't set to maximized after we've resized.
     if (_maximized_horz || _maximized_vert) {
@@ -1271,7 +1304,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
 
     if (outline) {
         moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
-        _scr->ungrabServer(true);
+        X11::ungrabServer(true);
     }
 }
 
@@ -1337,8 +1370,8 @@ Frame::moveToEdge(OrientationType ori)
     Geometry head, real_head;
 
     head_nr = getNearestHead();
-    _scr->getHeadInfo(head_nr, real_head);
-    _scr->getHeadInfoWithEdge(head_nr, head);
+    X11::getHeadInfo(head_nr, real_head);
+    X11::getHeadInfoWithEdge(head_nr, head);
 
     switch (ori) {
     case TOP_LEFT:
@@ -1423,11 +1456,6 @@ Frame::updateInactiveChildInfo(void)
 void
 Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
 {
-    // we don't want to maximize transients
-    if (_client->isTransient()) {
-        return;
-    }
-
     setShaded(STATE_UNSET);
     setStateFullscreen(STATE_UNSET);
 
@@ -1442,7 +1470,7 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
     XSizeHints *size_hint = _client->getXSizeHints(); // convenience
 
     Geometry head;
-    _scr->getHeadInfoWithEdge(getNearestHead(), head);
+    X11::getHeadInfoWithEdge(getNearestHead(), head);
 
     int max_x, max_r, max_y, max_b;
     max_x = head.x;
@@ -1554,7 +1582,7 @@ Frame::setStateFullscreen(StateAction sa)
 
         Geometry head;
         uint nr = getNearestHead();
-        _scr->getHeadInfo(nr, head);
+        X11::getHeadInfo(nr, head);
 
         _gm = head;
     }
@@ -1737,7 +1765,6 @@ Frame::setStateMarked(StateAction sa, Client *client)
     renderTitle();
 }
 
-#ifdef OPACITY
 void
 Frame::setStateOpaque(StateAction sa)
 {
@@ -1747,7 +1774,6 @@ Frame::setStateOpaque(StateAction sa)
     _client->setOpaque(!_opaque);
     setOpaque(!_opaque);
 }
-#endif // OPACITY
 // STATE actions end
 
 //! @brief
@@ -1755,7 +1781,7 @@ void
 Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
 {
     int f_r, f_b;
-    int x, y, h, w, r, b;
+    int x, y, r, b;
 
     f_r = getRX();
     f_b = getBY();
@@ -1768,8 +1794,6 @@ Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
 
         x = (*it)->getX();
         y = (*it)->getY();
-        h = (*it)->getHeight();
-        w = (*it)->getWidth();
         r = (*it)->getRX();
         b = (*it)->getBY();
 
@@ -1795,7 +1819,7 @@ void
 Frame::growDirection(uint direction)
 {
     Geometry head;
-    _scr->getHeadInfoWithEdge(getNearestHead(), head);
+    X11::getHeadInfoWithEdge(getNearestHead(), head);
 
     switch (direction) {
     case DIRECTION_UP:
@@ -2047,10 +2071,10 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
             if (ev->value_mask&CWSibling) {
                 switch(ev->detail) {
                 case Above:
-                    Workspaces::instance()->stackAbove(this, ev->above);
+                    Workspaces::stackAbove(this, ev->above);
                     break;
                 case Below:
-                    Workspaces::instance()->stackBelow(this, ev->above);
+                    Workspaces::stackBelow(this, ev->above);
                     break;
                 case TopIf:
                 case BottomIf:
@@ -2119,10 +2143,10 @@ Frame::isRequestGeometryFullscreen(XConfigureRequestEvent *ev, Client *client)
     bool is_fullscreen = false;
     if (! client->isCfgDeny(CFG_DENY_SIZE)
         && ! client->isCfgDeny(CFG_DENY_POSITION)) {
-        int nearest_head = _scr->getNearestHead(ev->x, ev->y);
+        int nearest_head = X11::getNearestHead(ev->x, ev->y);
         Geometry gm_request(ev->x, ev->y, ev->width, ev->height);
-        Geometry gm_screen(_scr->getScreenGeometry());
-        Geometry gm_head(_scr->getHeadGeometry(nearest_head));
+        Geometry gm_screen(X11::getScreenGeometry());
+        Geometry gm_head(X11::getHeadGeometry(nearest_head));
 
         if (gm_request == gm_screen || gm_request == gm_head) {
             is_fullscreen = true;
@@ -2159,9 +2183,9 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
             // If we aren't mapped we check if we make sure we're on the right
             // workspace and then map the window.
             if (! _mapped) {
-                if (_workspace != Workspaces::instance()->getActive() &&
+                if (_workspace != Workspaces::getActive() &&
                         !isSticky()) {
-                    Workspaces::instance()->setWorkspace(_workspace, false);
+                    Workspaces::setWorkspace(_workspace, false);
                 }
                 mapWindow();
             }
@@ -2232,7 +2256,13 @@ Frame::handleStateAtom(StateAction sa, Atom atom, Client *client)
         client->setStateSkip(sa, SKIP_PAGER);
         break;
     case STATE_DEMANDS_ATTENTION:
-        client->setStateDemandsAttention(sa, true);
+        bool ostate = client->demandsAttention();
+        bool nstate = (sa == STATE_SET || (sa == STATE_TOGGLE && ! ostate));
+        client->setDemandsAttention(nstate);
+        if (ostate != nstate) {
+            _attention += !!nstate - !!ostate;
+            setDecor(_attention?DEFAULT_DECOR_NAME_ATTENTION:getClientDecorName(_client));
+        }
         break;
     }
 }
@@ -2297,6 +2327,14 @@ Frame::handlePropertyChange(XPropertyEvent *ev, Client *client)
         client->getWMNormalHints();
     } else if (ev->atom == XA_WM_TRANSIENT_FOR) {
         client->getTransientForHint();
+    } else if (ev->atom == Atoms::getAtom(WM_HINTS)) {
+        bool nstate, ostate = client->demandsAttention();
+        client->getWMHints();
+        nstate = client->demandsAttention();
+        if (ostate != nstate) {
+            _attention += !!nstate - !!ostate;
+            setDecor(_attention?DEFAULT_DECOR_NAME_ATTENTION:getClientDecorName(_client));
+        }
     }
 }
 
@@ -2311,7 +2349,7 @@ Frame::handleTitleChange(Client *client)
     client->readName();
 
     bool require_render = true;
-    if (client == _client) {
+    if (client == _client && ! _attention) {
         string new_decor_name(getClientDecorName(client));
         if (new_decor_name != _decor_name) {
             require_render = ! setDecor(new_decor_name);
