@@ -1,23 +1,21 @@
 //
 // FrameListMenu.cc for pekwm
-// Copyright © 2002-2007 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
-// $Id$
-//
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
-#ifdef MENUS
-
 #include <algorithm>
 #include <cstdio>
+#include <cwchar>
 #include <iostream>
 
+#include "Compat.hh"
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "PMenu.hh"
@@ -36,19 +34,19 @@ using std::list;
 using std::string;
 using std::vector;
 using std::wstring;
+using std::swprintf;
 
 //! @brief FrameListMenu constructor.
-//! @param scr Pointer to PScreen.
 //! @param theme Pointer to Theme
 //! @param type Type of menu.
 //! @param title Title of menu.
 //! @param name Name of menu
 //! @param decor_name Decor name, defaults to MENU
-FrameListMenu::FrameListMenu(PScreen *scr, Theme *theme,
+FrameListMenu::FrameListMenu(Theme *theme,
                              MenuType type,
                              const std::wstring &title, const std::string &name,
                              const std::string &decor_name)
-    : WORefMenu(scr, theme, title, name, decor_name)
+    : WORefMenu(theme, title, name, decor_name)
 {
     _menu_type = type;
 }
@@ -72,7 +70,9 @@ FrameListMenu::mapWindow(void)
 
 // END - PWinObj interface.
 
-//! @brief
+/**
+ * Execute item execution.
+ */
 void
 FrameListMenu::handleItemExec(PMenu::Item *item)
 {
@@ -80,29 +80,25 @@ FrameListMenu::handleItemExec(PMenu::Item *item)
         return;
     }
 
-    if (! PWinObj::windowObjectExists(_wo_ref)) {
-        _wo_ref = NULL;
-    }
-    if (! PWinObj::windowObjectExists(item->getWORef())) {
-        item->setWORef(NULL);
-    }
-
+    Client *item_client = dynamic_cast<Client*>(item->getWORef());
+    Client *wo_ref_client = dynamic_cast<Client*>(getWORef());
+    
     switch (_menu_type) {
     case GOTOMENU_TYPE:
     case GOTOCLIENTMENU_TYPE:
-        handleGotomenu(dynamic_cast<Client*>(item->getWORef()));
+        handleGotomenu(item_client);
         break;
     case ICONMENU_TYPE:
-        handleIconmenu(dynamic_cast<Client*>(item->getWORef()));
+        handleIconmenu(item_client);
         break;
     case ATTACH_CLIENT_TYPE:
     case ATTACH_FRAME_TYPE:
-        handleAttach(dynamic_cast<Client*>(_wo_ref), dynamic_cast<Client*>(item->getWORef()),
+        handleAttach(wo_ref_client, item_client,
                      (_menu_type == ATTACH_FRAME_TYPE));
         break;
     case ATTACH_CLIENT_IN_FRAME_TYPE:
     case ATTACH_FRAME_IN_FRAME_TYPE:
-        handleAttach(dynamic_cast<Client*>(item->getWORef()), dynamic_cast<Client*>(_wo_ref),
+        handleAttach(item_client, wo_ref_client,
                      (_menu_type == ATTACH_FRAME_IN_FRAME_TYPE));
         break;
     default:
@@ -139,8 +135,8 @@ FrameListMenu::updateFrameListMenu(void)
     // if we have 1 workspace, we won't put an workspace indicator
     buf[0] = '\0';
 
-    for (uint i = 0; i < Workspaces::instance()->size(); ++i) {
-        if (Workspaces::instance()->size() > 1) {
+    for (uint i = 0; i < Workspaces::size(); ++i) {
+        if (Workspaces::size() > 1) {
             swprintf(buf, 16, L"<%d> ", i + 1);
         }
 
@@ -148,7 +144,7 @@ FrameListMenu::updateFrameListMenu(void)
             if (((*it)->getWorkspace() == i) && // sort by workspace
                     // don't include ourselves if we're not doing a gotoclient menu
                     ((_menu_type != GOTOCLIENTMENU_TYPE)
-                     ? ((*it)->getActiveChild() != _wo_ref)
+                     ? ((*it)->getActiveChild() != getWORef())
                      : true) &&
                     (show_iconified_only
                      ? (*it)->isIconified()
@@ -243,8 +239,8 @@ FrameListMenu::handleGotomenu(Client *client)
 
     // make sure it's on correct workspace
     if (! frame->isSticky() &&
-            (frame->getWorkspace() != Workspaces::instance()->getActive())) {
-        Workspaces::instance()->setWorkspace(frame->getWorkspace(), false);
+            (frame->getWorkspace() != Workspaces::getActive())) {
+        Workspaces::setWorkspace(frame->getWorkspace(), false);
     }
     // make sure it isn't hidden
     if (! frame->isMapped()) {
@@ -266,8 +262,8 @@ FrameListMenu::handleIconmenu(Client *client)
     Frame *frame = static_cast<Frame*>(client->getParent());
 
     // make sure it's on the current workspace
-    if (frame->getWorkspace() != Workspaces::instance()->getActive()) {
-        frame->setWorkspace(Workspaces::instance()->getActive());
+    if (frame->getWorkspace() != Workspaces::getActive()) {
+        frame->setWorkspace(Workspaces::getActive());
     }
 
     frame->raise();
@@ -297,5 +293,3 @@ FrameListMenu::handleAttach(Client *client_to, Client *client_from, bool frame)
         frame_to->giveInputFocus();
     }
 }
-
-#endif // MENUS
