@@ -13,13 +13,12 @@
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "PMenu.hh"
-#include "PScreen.hh"
 //#include "Action.hh"
 #include "HarbourMenu.hh"
 #include "DockApp.hh"
 
 //! @brief HarbourMenu constructor
-HarbourMenu::HarbourMenu(PScreen *scr, Theme *theme, Harbour *harbour)
+HarbourMenu::HarbourMenu(Theme *theme, Harbour *harbour)
     : PMenu(theme, L"Harbour", "HARBOUR"),
       _harbour(harbour), _dockapp(0)
 {
