@@ -1,6 +1,6 @@
 //
 // KeyGrabber.cc for pekwm
-// Copyright © 2003-2008 Claes Nästén <me@pekdon.net>
+// Copyright © 2003-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -13,7 +13,7 @@
 #include "KeyGrabber.hh"
 
 #include "Config.hh"
-#include "PScreen.hh"
+#include "x11.hh"
 #include "CfgParser.hh"
 #include "Util.hh"
 
@@ -90,25 +90,22 @@ KeyGrabber::Chain::findAction(XKeyEvent *ev)
 }
 
 //! @brief KeyGrabber constructor
-KeyGrabber::KeyGrabber(PScreen *scr)
-    : _scr(scr), _keygrabber_mtime(0),
-#ifdef MENUS
-      _menu_chain(0, 0),
-#endif // MENUS
+KeyGrabber::KeyGrabber(void)
+    : _menu_chain(0, 0),
       _global_chain(0, 0), _moveresize_chain(0, 0),
       _input_dialog_chain(0, 0)
 {
 #ifdef DEBUG
     if (_instance) {
         cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "KeyGrabber(" << this << ")::KeyGrabber(" << scr << ")" << endl
-             << " *** _instance allready set: " << _instance << endl;
+             << "KeyGrabber(" << this << ")::KeyGrabber()" << endl
+             << " *** _instance already set: " << _instance << endl;
     }
 #endif // DEBUG
     _instance = this;
 
-    _num_lock = _scr->getNumLock();
-    _scroll_lock = _scr->getScrollLock();
+    _num_lock = X11::getNumLock();
+    _scroll_lock = X11::getScrollLock();
 }
 
 //! @brief Destructor for KeyGrabber class
@@ -121,22 +118,28 @@ KeyGrabber::~KeyGrabber(void)
 //! If _global_keys holds any keygrabs they will be flushed before
 //! reloading the new keybindings.
 bool
-KeyGrabber::load(const std::string &file)
+KeyGrabber::load(const std::string &file, bool force)
 {
-    if (! Util::requireReload(_keygrabber_path, file, _keygrabber_mtime)) {
+    if (! force && ! Util::requireReload(_cfg_state, file)) {
         return false;
     }
 
     CfgParser key_cfg;
     if (! key_cfg.parse(file, CfgParserSource::SOURCE_FILE)) {
-        _keygrabber_path = SYSCONFDIR "/keys";
-        if (! key_cfg.parse(_keygrabber_path, CfgParserSource::SOURCE_FILE)) {
+        _cfg_state.clear();
+        if (! key_cfg.parse(SYSCONFDIR "/keys", CfgParserSource::SOURCE_FILE, true)) {
             cerr << __FILE__ << "@" << __LINE__ << "Error: no keyfile at " << file
-                 << " or " << _keygrabber_path << endl;
+                 << " or " << SYSCONFDIR "/keys" << endl;
             return false;
         }
     }
 
+    if (key_cfg.is_dynamic_content()) {
+        _cfg_state.clear();
+    } else {
+        _cfg_state = key_cfg.get_file_list();
+    }
+
     CfgParser::Entry *section;
 
     section = key_cfg.get_entry_root()->find_section("GLOBAL");
@@ -164,13 +167,11 @@ KeyGrabber::load(const std::string &file)
         parseInputDialogChain (section, &_input_dialog_chain);
     }
 
-#ifdef MENUS
     section = key_cfg.get_entry_root ()->find_section("MENU");
     if (section) {
         _menu_chain.unload();
         parseMenuChain(section, &_menu_chain);
     }
-#endif // MENUS
 
     return true;
 }
@@ -241,7 +242,6 @@ KeyGrabber::parseInputDialogChain(CfgParser::Entry *section, KeyGrabber::Chain *
     }
 }
 
-#ifdef MENUS
 //! @brief Parses chain, getting actions as MenuEvents
 void
 KeyGrabber::parseMenuChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
@@ -263,7 +263,6 @@ KeyGrabber::parseMenuChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
         }
     }
 }
-#endif // MENUS
 
 //! @brief Grabs all the keybindings in _keybindings on the Window win.
 //! @param win Window to grab the keys on.
@@ -288,7 +287,7 @@ KeyGrabber::grabKeys(Window win)
 void
 KeyGrabber::grabKey(Window win, uint mod, uint key)
 {
-    Display *dpy = _scr->getDpy(); // convenience
+    Display *dpy = X11::getDpy(); // convenience
 
     XGrabKey(dpy, key, mod, win, true, GrabModeAsync, GrabModeAsync);
     XGrabKey(dpy, key, mod|LockMask, win, true, GrabModeAsync, GrabModeAsync);
@@ -318,7 +317,7 @@ KeyGrabber::grabKey(Window win, uint mod, uint key)
 void
 KeyGrabber::ungrabKeys(Window win)
 {
-    XUngrabKey(_scr->getDpy(), AnyKey, AnyModifier, win);
+    XUngrabKey(X11::getDpy(), AnyKey, AnyModifier, win);
 }
 
 //! @brief Tries to match the XKeyEvent to an usefull action and return it
@@ -329,20 +328,21 @@ KeyGrabber::findAction(XKeyEvent *ev, KeyGrabber::Chain *chain)
     if (! ev) {
         return 0;
     }
-    ev->state &= ~_num_lock & ~_scroll_lock & ~LockMask;
+
+    X11::stripStateModifiers(&ev->state);
 
     ActionEvent *action = 0;
     KeyGrabber::Chain *sub_chain = _global_chain.findChain(ev);
-    if (sub_chain && _scr->grabKeyboard(_scr->getRoot())) {
+    if (sub_chain && X11::grabKeyboard(X11::getRoot())) {
         XEvent c_ev;
         KeyGrabber::Chain *last_chain;
         bool exit = false;
 
         while (! exit) {
-            XMaskEvent(_scr->getDpy(), KeyPressMask, &c_ev);
-            c_ev.xkey.state &= ~_num_lock & ~_scroll_lock & ~LockMask;
+            XMaskEvent(X11::getDpy(), KeyPressMask, &c_ev);
+            X11::stripStateModifiers(&c_ev.xkey.state);
 
-            if (IsModifierKey(XKeycodeToKeysym(_scr->getDpy(),
+            if (IsModifierKey(XKeycodeToKeysym(X11::getDpy(),
                                                c_ev.xkey.keycode, 0))) {
                 // do nothing
             } else  if ((last_chain = sub_chain->findChain(&c_ev.xkey))) {
@@ -353,7 +353,7 @@ KeyGrabber::findAction(XKeyEvent *ev, KeyGrabber::Chain *chain)
             }
         }
 
-        _scr->ungrabKeyboard();
+        X11::ungrabKeyboard();
     } else {
         action = chain->findAction(ev);
     }
@@ -367,12 +367,10 @@ KeyGrabber::findAction(XKeyEvent *ev, PWinObj::Type type)
 {
     ActionEvent *ae = 0;
 
-#ifdef MENUS
     if (type == PWinObj::WO_MENU) {
         ae = findAction(ev, &_menu_chain);
     }
-#endif // MENUS
-    if (type == PWinObj::WO_CMD_DIALOG) {
+    if (type == PWinObj::WO_CMD_DIALOG || type == PWinObj::WO_SEARCH_DIALOG) {
         ae = findAction(ev, &_input_dialog_chain);
     }
 
