@@ -1,15 +1,13 @@
 //
 // PWinObj.cc for pekwm
-// Copyright © 2003-2007 Claes Nästen <me{@}pekdon{.}net>
+// Copyright © 2003-2009 Claes Nästen <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
-// $Id$
-//
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif // HAVE_CONFIG_H
 
 #include <algorithm>
@@ -17,6 +15,8 @@
 
 #include "PWinObj.hh"
 
+#include "Atoms.hh"
+
 using std::cerr;
 using std::endl;
 using std::find;
@@ -29,13 +29,14 @@ vector<PWinObj*> PWinObj::_wo_list = vector<PWinObj*>();
 map<Window, PWinObj*> PWinObj::_wo_map = map<Window, PWinObj*>();
 
 //! @brief PWinObj constructor.
-PWinObj::PWinObj(Display *dpy)
-    : _dpy(dpy), _window(None),
+PWinObj::PWinObj(void)
+    : _window(None),
       _parent(0), _type(WO_NO_TYPE),
-      _workspace(0), _layer(LAYER_NORMAL),
-      _mapped(false), _iconified(false), _hidden(false),
-      _focused(false), _sticky(false),
+      _opaque(true), _workspace(0), _layer(LAYER_NORMAL),
+      _mapped(false), _iconified(false), 
+      _hidden(false), _focused(false), _sticky(false),
       _focusable(true)
+      
 {
 }
 
@@ -45,6 +46,7 @@ PWinObj::~PWinObj(void)
     if (_focused_wo == this) {
         _focused_wo = 0;
     }
+    notifyObservers(0);
 }
 
 //! @brief Associates Window with PWinObj
@@ -64,6 +66,33 @@ PWinObj::removeChildWindow(Window win)
     }
 }
 
+//! @brief Sets the desired opacity values for focused/unfocused states
+void
+PWinObj::setOpacity(uint focused, uint unfocused, bool enabled)
+{
+    _opacity.focused = focused;
+    _opacity.unfocused = unfocused;
+    _opaque = !enabled;
+    updateOpacity();
+}
+
+//! @brief Updates the opacity hint based on focused state
+void
+PWinObj::updateOpacity(void)
+{
+    uint opacity;
+    if (_opaque) {
+        opacity = EWMH_OPAQUE_WINDOW;
+    } else {
+        opacity = isFocused()?_opacity.focused:_opacity.unfocused;
+    }
+
+    if (_opacity.current != opacity) {
+        _opacity.current = opacity;
+        AtomUtil::setLong(_window, Atoms::getAtom(NET_WM_WINDOW_OPACITY), opacity);
+    }
+}
+
 //! @brief Maps the window and sets _mapped to true.
 void
 PWinObj::mapWindow(void)
@@ -74,7 +103,7 @@ PWinObj::mapWindow(void)
     _mapped = true;
     _iconified = false;
 
-    XMapWindow(_dpy, _window);
+    X11::mapWindow(_window);
 }
 
 //! @brief Maps the window and raises it
@@ -87,7 +116,7 @@ PWinObj::mapWindowRaised(void)
     _mapped = true;
     _iconified = false;
 
-    XMapRaised(_dpy, _window);
+    XMapRaised(X11::getDpy(), _window);
 }
 
 //! @brief Unmaps the window and sets _mapped to false.
@@ -103,7 +132,7 @@ PWinObj::unmapWindow(void)
     // Make sure unmapped windows drops focus
     setFocused(false);
 
-    XUnmapWindow(_dpy, _window);
+    X11::unmapWindow(_window);
 }
 
 //! @brief Only sets _iconified to true.
@@ -133,7 +162,7 @@ PWinObj::move(int x, int y)
     _gm.x = x;
     _gm.y = y;
 
-    XMoveWindow(_dpy, _window, _gm.x, _gm.y);
+    XMoveWindow(X11::getDpy(), _window, _gm.x, _gm.y);
 }
 
 //! @brief Resizes the window and updates _gm.
@@ -153,7 +182,7 @@ PWinObj::resize(uint width, uint height)
     _gm.width = width;
     _gm.height = height;
 
-    XResizeWindow(_dpy, _window, _gm.width, _gm.height);
+    XResizeWindow(X11::getDpy(), _window, _gm.width, _gm.height);
 }
 
 //! @brief Move and resize window in one call.
@@ -179,7 +208,7 @@ PWinObj::moveResize(int x, int y, uint width, uint height)
     _gm.width = width;
     _gm.height = height;
 
-    XMoveResizeWindow(_dpy, _window, _gm.x, _gm.y, _gm.width, _gm.height);
+    XMoveResizeWindow(X11::getDpy(), _window, _gm.x, _gm.y, _gm.width, _gm.height);
 }
 
 //! @brief Only sets _workspace to workspace.
@@ -196,11 +225,12 @@ PWinObj::setLayer(uint layer)
     _layer = layer;
 }
 
-//! @brief Only sets _focused to focused.
+//! @brief Sets _focused to focused and updates opacity as needed.
 void
 PWinObj::setFocused(bool focused)
 {
     _focused = focused;
+    updateOpacity();
 }
 
 //! @brief Only sets _sticky to sticky.
@@ -210,6 +240,14 @@ PWinObj::setSticky(bool sticky)
     _sticky = sticky;
 }
 
+//! @brief Updates opaque state
+void
+PWinObj::setOpaque(bool opaque)
+{
+    _opaque = opaque;
+    updateOpacity();
+}
+
 //! @brief Only sets _hidden to hidden.
 void
 PWinObj::setHidden(bool hidden)
@@ -231,7 +269,7 @@ PWinObj::giveInputFocus(void)
         return;
     }
 
-    XSetInputFocus(_dpy, _window, RevertToPointerRoot, CurrentTime);
+    XSetInputFocus(X11::getDpy(), _window, RevertToPointerRoot, CurrentTime);
 }
 
 //! @brief Reparents and sets _parent member
@@ -239,7 +277,7 @@ void
 PWinObj::reparent(PWinObj *wo, int x, int y)
 {
     _parent = wo;
-    XReparentWindow(_dpy, _window, wo->getWindow(), x, y);
+    XReparentWindow(X11::getDpy(), _window, wo->getWindow(), x, y);
 }
 
 //! @brief Get required size to hold content for window
