@@ -20,7 +20,6 @@
 #include "Client.hh"
 #include "WindowManager.hh"
 
-#include "PScreen.hh"
 #include "ScreenResources.hh"
 #include "ActionHandler.hh"
 #include "AutoProperties.hh"
@@ -34,6 +33,7 @@
 #include "TextureHandler.hh"
 #include "Workspaces.hh"
 #include "Util.hh"
+#include "x11.hh"
 
 #include "RegexString.hh"
 
@@ -164,12 +164,12 @@ WindowManager::destroy(void)
 WindowManager::WindowManager(const std::string &command_line,
                              const std::string &config_file)
         :
-        _screen(0), _screen_resources(0),
+        _screen_resources(0),
         _keygrabber(0),
         _config(0), _color_handler(0),
         _font_handler(0), _texture_handler(0),
         _theme(0), _action_handler(0),
-        _autoproperties(0), _workspaces(0),
+        _autoproperties(0),
         _harbour(0),
         _cmd_dialog(0), _search_dialog(0),
         _status_window(0), _workspace_indicator(0),
@@ -212,7 +212,7 @@ WindowManager::~WindowManager(void)
     delete _action_handler;
     delete _autoproperties;
     delete _keygrabber;
-    delete _workspaces;
+    Workspaces::free();
     delete _config;
     delete _theme;
     delete _color_handler;
@@ -220,13 +220,7 @@ WindowManager::~WindowManager(void)
     delete _texture_handler;
     delete _screen_resources;
 
-    if (_screen) {
-        Display *dpy = _screen->getDpy();
-
-        delete _screen;
-
-        XCloseDisplay(dpy);
-    }
+    X11::destruct();
 }
 
 //! @brief Checks if the start file is executable then execs it.
@@ -256,25 +250,24 @@ WindowManager::cleanup(void)
         (*it_f)->updateInactiveChildInfo();
     }
 
-    _harbour->removeAllDockApps();
+    if (_harbour) {
+        _harbour->removeAllDockApps();
+    }
 
     // To preserve stacking order when destroying the frames, we go through
     // the PWinObj list from the Workspaces and put all Frames into our own
     // list, then we delete the frames in order.
-    if (_workspaces) {
-        list<Frame*> frame_list;
-
-        list<PWinObj*>::iterator it_w(_workspaces->begin());
-        for (; it_w != _workspaces->end(); ++it_w) {
-            if ((*it_w)->getType() == PWinObj::WO_FRAME) {
-                frame_list.push_back(static_cast<Frame*>(*it_w));
-            }
+    list<Frame*> frame_list;
+    list<PWinObj*>::iterator it_w(Workspaces::begin());
+    for (; it_w != Workspaces::end(); ++it_w) {
+        if ((*it_w)->getType() == PWinObj::WO_FRAME) {
+            frame_list.push_back(static_cast<Frame*>(*it_w));
         }
+    }
 
-        // Delete all Frames. This reparents the Clients.
-        for (it_f = frame_list.begin(); it_f != frame_list.end(); ++it_f) {
-            delete *it_f;
-        }
+    // Delete all Frames. This reparents the Clients.
+    for (it_f = frame_list.begin(); it_f != frame_list.end(); ++it_f) {
+        delete *it_f;
     }
 
     // Delete all Clients.
@@ -285,14 +278,14 @@ WindowManager::cleanup(void)
     }
 
     if (_keygrabber) {
-        _keygrabber->ungrabKeys(_screen->getRoot());
+        _keygrabber->ungrabKeys(X11::getRoot());
     }
 
     // destroy screen edge
     screenEdgeDestroy();
 
-    XInstallColormap(_screen->getDpy(), _screen->getColormap());
-    XSetInputFocus(_screen->getDpy(), PointerRoot, RevertToPointerRoot, CurrentTime);
+    XInstallColormap(X11::getDpy(), X11::getColormap());
+    XSetInputFocus(X11::getDpy(), PointerRoot, RevertToPointerRoot, CurrentTime);
 }
 
 /**
@@ -310,21 +303,19 @@ WindowManager::setupDisplay(bool replace)
     }
 
     // Setup screen, init atoms and claim the display.
-    _screen = new PScreen(dpy, _config->isHonourRandr());
-
-    PWinObj::setDisplay(dpy);
+    X11::init(dpy, _config->isHonourRandr());
 
     Atoms::init();
 
     try {
         // Create hint window _before_ root window.
-        _hint_wo = new HintWO(_screen->getRoot(), replace);
+        _hint_wo = new HintWO(X11::getRoot(), replace);
     } catch (string &ex) {
         return false;
     }
 
     // Create root PWinObj
-    _root_wo = new RootWO(_screen->getRoot());
+    _root_wo = new RootWO(X11::getRoot());
     PWinObj::setRootPWinObj(_root_wo);
 
     _color_handler = new ColorHandler(dpy);
@@ -337,14 +328,15 @@ WindowManager::setupDisplay(bool replace)
 
     // load colors, fonts
     _screen_resources = new ScreenResources();
-    _theme = new Theme(_screen);
+    _theme = new Theme;
 
     _autoproperties = new AutoProperties();
     _autoproperties->load();
 
-    _workspaces = new Workspaces(_config->getWorkspaces(), _config->getWorkspacesPerRow());
+    Workspaces::setSize(_config->getWorkspaces());
+    Workspaces::setPerRow(_config->getWorkspacesPerRow());
 
-    _harbour = new Harbour(_screen, _theme, _workspaces);
+    _harbour = new Harbour(_theme);
 
     MenuHandler::init(_theme);
 
@@ -353,16 +345,16 @@ WindowManager::setupDisplay(bool replace)
     _status_window = new StatusWindow(_theme);
     _workspace_indicator = new WorkspaceIndicator(_theme, _timer_action);
 
-    XDefineCursor(dpy, _screen->getRoot(),
+    XDefineCursor(dpy, X11::getRoot(),
                   _screen_resources->getCursor(ScreenResources::CURSOR_ARROW));
 
 #ifdef HAVE_XRANDR
-    XRRSelectInput(dpy, _screen->getRoot(), RRScreenChangeNotifyMask|RRCrtcChangeNotifyMask);
+    XRRSelectInput(dpy, X11::getRoot(), RRScreenChangeNotifyMask|RRCrtcChangeNotifyMask);
 #endif // HAVE_XRANDR
 
-    _keygrabber = new KeyGrabber(_screen);
+    _keygrabber = new KeyGrabber;
     _keygrabber->load(_config->getKeyFile());
-    _keygrabber->grabKeys(_screen->getRoot());
+    _keygrabber->grabKeys(X11::getRoot());
 
     // Create screen edge windows
     screenEdgeCreate();
@@ -384,7 +376,7 @@ WindowManager::scanWindows(void)
     XWindowAttributes attr;
 
     // Lets create a list of windows on the display
-    XQueryTree(_screen->getDpy(), _screen->getRoot(),
+    XQueryTree(X11::getDpy(), X11::getRoot(),
                &d_win1, &d_win2, &wins, &num_wins);
     list<Window> win_list(wins, wins + num_wins);
     XFree(wins);
@@ -399,7 +391,7 @@ WindowManager::scanWindows(void)
             continue;
         }
 
-        XWMHints *wm_hints = XGetWMHints(_screen->getDpy(), *it);
+        XWMHints *wm_hints = XGetWMHints(X11::getDpy(), *it);
         if (wm_hints) {
             if ((wm_hints->flags&IconWindowHint) &&
                     (wm_hints->icon_window != *it)) {
@@ -417,13 +409,13 @@ WindowManager::scanWindows(void)
             continue;
         }
         
-        XGetWindowAttributes(_screen->getDpy(), *it, &attr);
+        XGetWindowAttributes(X11::getDpy(), *it, &attr);
         if (! attr.override_redirect && attr.map_state != IsUnmapped) {
-            XWMHints *wm_hints = XGetWMHints(_screen->getDpy(), *it);
+            XWMHints *wm_hints = XGetWMHints(X11::getDpy(), *it);
             if (wm_hints) {
                 if ((wm_hints->flags&StateHint) &&
                         (wm_hints->initial_state == WithdrawnState)) {
-                    _harbour->addDockApp(new DockApp(_screen, _theme, *it));
+                    _harbour->addDockApp(new DockApp(_theme, *it));
                 } else {
                     client = new Client(*it);
                     if (! client->isAlive()) {
@@ -441,7 +433,7 @@ WindowManager::scanWindows(void)
     }
 
     // Try to focus the ontop window, if no window we give root focus
-    PWinObj *wo = _workspaces->getTopWO(PWinObj::WO_FRAME);
+    PWinObj *wo = Workspaces::getTopWO(PWinObj::WO_FRAME);
     if (wo && wo->isMapped()) {
         wo->giveInputFocus();
     } else {
@@ -473,19 +465,19 @@ WindowManager::screenEdgeCreate(void)
 
     bool indent = Config::instance()->getScreenEdgeIndent();
 
-    _screen_edge_list.push_back(new EdgeWO(_screen->getRoot(), SCREEN_EDGE_LEFT,
+    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_LEFT,
                                            indent && (_config->getScreenEdgeSize(SCREEN_EDGE_LEFT) > 0)));
-    _screen_edge_list.push_back(new EdgeWO(_screen->getRoot(), SCREEN_EDGE_RIGHT,
+    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_RIGHT,
                                            indent && (_config->getScreenEdgeSize(SCREEN_EDGE_RIGHT) > 0)));
-    _screen_edge_list.push_back(new EdgeWO(_screen->getRoot(), SCREEN_EDGE_TOP,
+    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_TOP,
                                            indent && (_config->getScreenEdgeSize(SCREEN_EDGE_TOP) > 0)));
-    _screen_edge_list.push_back(new EdgeWO(_screen->getRoot(), SCREEN_EDGE_BOTTOM,
+    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_BOTTOM,
                                            indent && (_config->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) > 0)));
 
     // make sure the edge stays ontop
     list<EdgeWO*>::iterator it(_screen_edge_list.begin());
     for (; it != _screen_edge_list.end(); ++it) {
-        _workspaces->insert(*it);
+        Workspaces::insert(*it);
     }
 
     screenEdgeResize();
@@ -501,7 +493,7 @@ WindowManager::screenEdgeDestroy(void)
 
     list<EdgeWO*>::iterator it(_screen_edge_list.begin());
     for (; it != _screen_edge_list.end(); ++it) {
-        _workspaces->remove(*it);
+        Workspaces::remove(*it);
         delete *it;
     }
 }
@@ -520,26 +512,26 @@ WindowManager::screenEdgeResize(void)
     list<EdgeWO*>::iterator it(_screen_edge_list.begin());
 
     // Left edge
-    (*it)->moveResize(0, 0, l_size, _screen->getHeight());
+    (*it)->moveResize(0, 0, l_size, X11::getHeight());
     ++it;
 
     // Right edge
-    (*it)->moveResize(_screen->getWidth() - r_size, 0, r_size, _screen->getHeight());
+    (*it)->moveResize(X11::getWidth() - r_size, 0, r_size, X11::getHeight());
     ++it;
 
     // Top edge
-    (*it)->moveResize(l_size, 0, _screen->getWidth() - l_size - r_size, t_size);
+    (*it)->moveResize(l_size, 0, X11::getWidth() - l_size - r_size, t_size);
     ++it;
 
     // Bottom edge
-    (*it)->moveResize(l_size, _screen->getHeight() - b_size, _screen->getWidth() - l_size - r_size, b_size);
+    (*it)->moveResize(l_size, X11::getHeight() - b_size, X11::getWidth() - l_size - r_size, b_size);
 
     for (it = _screen_edge_list.begin(); it != _screen_edge_list.end(); ++it) {
       (*it)->configureStrut(_config->getScreenEdgeIndent()
                             && (_config->getScreenEdgeSize((*it)->getEdge()) > 0));
     }
 
-    _screen->updateStrut();
+    X11::updateStrut();
 }
 
 void
@@ -595,9 +587,9 @@ WindowManager::doReloadConfig(void)
     }
 
     // Update what might have changed in the cfg touching the hints
-    _workspaces->setSize(_config->getWorkspaces());
-    _workspaces->setPerRow(_config->getWorkspacesPerRow());
-    _workspaces->setNames();
+    Workspaces::setSize(_config->getWorkspaces());
+    Workspaces::setPerRow(_config->getWorkspacesPerRow());
+    Workspaces::setNames();
 
     // Flush pixmap cache and set size
     _screen_resources->getPixmapHandler()->setCacheSize(_config->getScreenPixmapCacheSize());
@@ -616,7 +608,7 @@ WindowManager::doReloadConfig(void)
     screenEdgeResize();
     screenEdgeMapUnmap();
 
-    _screen->updateStrut();
+    X11::updateStrut();
 }
 
 /**
@@ -658,8 +650,8 @@ WindowManager::doReloadKeygrabber(bool force)
         return;
     }
 
-    _keygrabber->ungrabKeys(_screen->getRoot());
-    _keygrabber->grabKeys(_screen->getRoot());
+    _keygrabber->ungrabKeys(X11::getRoot());
+    _keygrabber->grabKeys(X11::getRoot());
 
     // Regrab keys and buttons
     list<Client*>::iterator c_it(Client::client_begin());
@@ -745,7 +737,7 @@ WindowManager::doEventLoop(void)
         }
 
         // Get next event, drop event handling if none was given
-        if (_screen->getNextEvent(ev)) {
+        if (X11::getNextEvent(ev)) {
             switch (ev.type) {
             case MapRequest:
                 handleMapRequestEvent(&ev.xmaprequest);
@@ -767,7 +759,7 @@ WindowManager::doEventLoop(void)
                 handleColormapEvent(&ev.xcolormap);
                 break;
             case PropertyNotify:
-                _screen->setLastEventTime(ev.xproperty.time);
+                X11::setLastEventTime(ev.xproperty.time);
                 handlePropertyEvent(&ev.xproperty);
                 break;
             case MappingNotify:
@@ -779,30 +771,30 @@ WindowManager::doEventLoop(void)
       
             case KeyPress:
             case KeyRelease:
-                _screen->setLastEventTime(ev.xkey.time);
+                X11::setLastEventTime(ev.xkey.time);
                 handleKeyEvent(&ev.xkey);
                 break;
 
             case ButtonPress:
-                _screen->setLastEventTime(ev.xbutton.time);
+                X11::setLastEventTime(ev.xbutton.time);
                 handleButtonPressEvent(&ev.xbutton);
                 break;
             case ButtonRelease:
-                _screen->setLastEventTime(ev.xbutton.time);
+                X11::setLastEventTime(ev.xbutton.time);
                 handleButtonReleaseEvent(&ev.xbutton);
                 break;
 
             case MotionNotify:
-                _screen->setLastEventTime(ev.xmotion.time);
+                X11::setLastEventTime(ev.xmotion.time);
                 handleMotionEvent(&ev.xmotion);
                 break;
 
             case EnterNotify:
-                _screen->setLastEventTime(ev.xcrossing.time);
+                X11::setLastEventTime(ev.xcrossing.time);
                 handleEnterNotify(&ev.xcrossing);
                 break;
             case LeaveNotify:
-                _screen->setLastEventTime(ev.xcrossing.time);
+                X11::setLastEventTime(ev.xcrossing.time);
                 handleLeaveNotify(&ev.xcrossing);
                 break;
             case FocusIn:
@@ -820,12 +812,12 @@ WindowManager::doEventLoop(void)
 
             default:
 #ifdef HAVE_SHAPE
-                if (_screen->hasExtensionShape() && (ev.type == _screen->getEventShape())) {
+                if (X11::hasExtensionShape() && (ev.type == X11::getEventShape())) {
                     handleShapeEvent(&ev.xany);
                 }
 #endif // HAVE_SHAPE
 #ifdef HAVE_XRANDR
-                if (_screen->hasExtensionXRandr() && (ev.type == _screen->getEventXRandr())) {
+                if (X11::hasExtensionXRandr() && (ev.type == X11::getEventXRandr())) {
                     handleXRandrEvent(reinterpret_cast<XRRNotifyEvent*>(&ev));
                 }
 #endif // HAVE_XRANDR
@@ -896,9 +888,9 @@ WindowManager::handleKeyEvent(XKeyEvent *ev)
 
     // flush Enter events caused by keygrabbing
     XEvent e;
-    while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, &e)) {
+    while (X11::checkTypedEvent(EnterNotify, &e)) {
         if (! e.xcrossing.send_event) {
-            _screen->setLastEventTime(e.xcrossing.time);
+            X11::setLastEventTime(e.xcrossing.time);
         }
     }
 }
@@ -908,9 +900,9 @@ void
 WindowManager::handleButtonPressEvent(XButtonEvent *ev)
 {
     // Clear event queue
-    while (XCheckTypedEvent(_screen->getDpy(), ButtonPress, (XEvent *) ev)) {
+    while (X11::checkTypedEvent(ButtonPress, (XEvent *) ev)) {
         if (! ev->send_event) {
-            _screen->setLastEventTime(ev->time);
+            X11::setLastEventTime(ev->time);
         }
     }
 
@@ -953,9 +945,9 @@ void
 WindowManager::handleButtonReleaseEvent(XButtonEvent *ev)
 {
     // Flush ButtonReleases
-    while (XCheckTypedEvent(_screen->getDpy(), ButtonRelease, (XEvent *) ev)) {
+    while (X11::checkTypedEvent(ButtonRelease, (XEvent *) ev)) {
         if (! ev->send_event) {
-            _screen->setLastEventTime(ev->time);
+            X11::setLastEventTime(ev->time);
         }
     }
 
@@ -1022,7 +1014,7 @@ WindowManager::handleConfigureRequestEvent(XConfigureRequestEvent *ev)
             wc.sibling = ev->above;
             wc.stack_mode = ev->detail;
 
-            XConfigureWindow(_screen->getDpy(), ev->window, ev->value_mask, &wc);
+            XConfigureWindow(X11::getDpy(), ev->window, ev->value_mask, &wc);
         }
     }
 }
@@ -1084,14 +1076,14 @@ WindowManager::handleMapRequestEvent(XMapRequestEvent *ev)
         wo->handleMapRequest(ev);
     } else {
         XWindowAttributes attr;
-        XGetWindowAttributes(_screen->getDpy(), ev->window, &attr);
+        XGetWindowAttributes(X11::getDpy(), ev->window, &attr);
         if (! attr.override_redirect) {
             // We need to figure out whether or not this is a dockapp.
-            XWMHints *wm_hints = XGetWMHints(_screen->getDpy(), ev->window);
+            XWMHints *wm_hints = XGetWMHints(X11::getDpy(), ev->window);
             if (wm_hints) {
                 if ((wm_hints->flags&StateHint) &&
                         (wm_hints->initial_state == WithdrawnState)) {
-                    _harbour->addDockApp(new DockApp(_screen, _theme, ev->window));
+                    _harbour->addDockApp(new DockApp(_theme, ev->window));
                 } else {
                     Client *client = new Client(ev->window, true);
                     if (! client->isAlive()) {
@@ -1171,9 +1163,9 @@ void
 WindowManager::handleEnterNotify(XCrossingEvent *ev)
 {
     // Clear event queue
-    while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, (XEvent *) ev)) {
+    while (X11::checkTypedEvent(EnterNotify, (XEvent *) ev)) {
         if (! ev->send_event) {
-            _screen->setLastEventTime(ev->time);
+            X11::setLastEventTime(ev->time);
         }
     }
 
@@ -1205,9 +1197,9 @@ void
 WindowManager::handleLeaveNotify(XCrossingEvent *ev)
 {
     // Clear event queue
-    while (XCheckTypedEvent(_screen->getDpy(), LeaveNotify, (XEvent *) ev)) {
+    while (X11::checkTypedEvent(LeaveNotify, (XEvent *) ev)) {
         if (! ev->send_event) {
-            _screen->setLastEventTime(ev->time);
+            X11::setLastEventTime(ev->time);
         }
     }
 
@@ -1254,14 +1246,14 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
             // all EnterNotify and LeaveNotify as they can interfere with
             // focusing if Sloppy or Follow like focus model is used.
             XEvent e_flush;
-            while (XCheckTypedEvent(_screen->getDpy(), EnterNotify, &e_flush)) {
+            while (X11::checkTypedEvent(EnterNotify, &e_flush)) {
                 if (! e_flush.xcrossing.send_event) {
-                    _screen->setLastEventTime(e_flush.xcrossing.time);
+                    X11::setLastEventTime(e_flush.xcrossing.time);
                 }
             }
-            while (XCheckTypedEvent(_screen->getDpy(), LeaveNotify, &e_flush)) {
+            while (X11::checkTypedEvent(LeaveNotify, &e_flush)) {
                 if (! e_flush.xcrossing.send_event) {
-                    _screen->setLastEventTime(e_flush.xcrossing.time);
+                    X11::setLastEventTime(e_flush.xcrossing.time);
                 }
             }
 
@@ -1274,7 +1266,7 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
                 } else {
                     focused_wo->setFocused(false);
                 }
-                _workspaces->setLastFocused(_workspaces->getActive(), focused_wo);
+                Workspaces::setLastFocused(Workspaces::getActive(), focused_wo);
             }
 
             PWinObj::setFocusedPWinObj(wo);
@@ -1283,9 +1275,11 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
                 wo->getParent()->setFocused(true);
                 _root_wo->setEwmhActiveWindow(wo->getWindow());
 
-                // update the MRU list
-                _mru_list.remove(wo->getParent());
-                _mru_list.push_back(wo->getParent());
+                // update the MRU list (except for skip focus windows, see #297)
+                if (! static_cast<Client*>(wo)->isSkip(SKIP_FOCUS_TOGGLE)) {
+                    _mru_list.remove(static_cast<Frame*>(wo->getParent()));
+                    _mru_list.push_back(static_cast<Frame*>(wo->getParent()));
+                }
             } else {
                 wo->setFocused(true);
                 _root_wo->setEwmhActiveWindow(None);
@@ -1299,7 +1293,7 @@ void
 WindowManager::handleFocusOutEvent(XFocusChangeEvent *ev)
 {
     // Get the last focus in event, no need to go through them all.
-    while (XCheckTypedEvent(_screen->getDpy(), FocusOut, (XEvent *) ev))
+    while (X11::checkTypedEvent(FocusOut, (XEvent *) ev))
         ;
 }
 
@@ -1311,18 +1305,18 @@ WindowManager::handleFocusOutEvent(XFocusChangeEvent *ev)
 void
 WindowManager::handleClientMessageEvent(XClientMessageEvent *ev)
 {
-    if (ev->window == _screen->getRoot()) {
+    if (ev->window == X11::getRoot()) {
         // root window messages
 
         if (ev->format == 32) {
 
             if (ev->message_type == Atoms::getAtom(NET_CURRENT_DESKTOP)) {
-                _workspaces->setWorkspace(ev->data.l[0], true);
+                Workspaces::setWorkspace(ev->data.l[0], true);
 
             } else if (ev->message_type ==
                        Atoms::getAtom(NET_NUMBER_OF_DESKTOPS)) {
                 if (ev->data.l[0] > 0) {
-                    _workspaces->setSize(ev->data.l[0]);
+                    Workspaces::setSize(ev->data.l[0]);
                 }
             }
         }
@@ -1352,10 +1346,10 @@ WindowManager::handleColormapEvent(XColormapEvent *ev)
 void
 WindowManager::handlePropertyEvent(XPropertyEvent *ev)
 {
-    if (ev->window == _screen->getRoot()) {
+    if (ev->window == X11::getRoot()) {
         if (ev->atom == Atoms::getAtom(NET_DESKTOP_NAMES)) {
             _root_wo->readEwmhDesktopNames();
-            _workspaces->setNames();
+            Workspaces::setNames();
         }
 
         return;
@@ -1374,7 +1368,7 @@ WindowManager::handleMappingEvent(XMappingEvent *ev)
 {
     if (ev->request == MappingKeyboard || ev->request == MappingModifier) {
         XRefreshKeyboardMapping(ev);
-        _screen->setLockKeys();
+        X11::setLockKeys();
         InputDialog::reloadKeysymMap();
         doReloadKeygrabber(true);
     }
@@ -1440,14 +1434,14 @@ WindowManager::handleXRandrScreenChangeEvent(XRRScreenChangeNotifyEvent *ev)
     cerr << __FILE__ << "@" << __LINE__ << ": WindowManager::handleXRandrScreenChangeEvent()" << endl;
 #endif // DEBUG
 
-    _screen->updateGeometry(ev->width, ev->height);
+    X11::updateGeometry(ev->width, ev->height);
     _harbour->updateGeometry();
     screenEdgeResize();
 
     // Make sure windows are visible after resize
     list<PDecor*>::iterator it(PDecor::pdecor_begin());
     for (; it != PDecor::pdecor_end(); ++it) {
-        _workspaces->placeWoInsideScreen(*it);
+        Workspaces::placeWoInsideScreen(*it);
     }
 }
 
@@ -1478,7 +1472,7 @@ WindowManager::findWOAndFocus(PWinObj *search)
 
     // search window object didn't exist, go through the MRU list
     if (! focus) {
-        list<PWinObj*>::reverse_iterator f_it = _mru_list.rbegin();
+        list<Frame *>::reverse_iterator f_it = _mru_list.rbegin();
         for (; ! focus  && (f_it != _mru_list.rend()); ++f_it) {
             if ((*f_it)->isMapped() && (*f_it)->isFocusable()) {
                 focus = *f_it;
