@@ -11,7 +11,6 @@
 #endif // HAVE_CONFIG_H
 
 #include "Workspaces.hh"
-#include "PScreen.hh"
 #include "Atoms.hh"
 #include "ParseUtil.hh"
 #include "Config.hh"
@@ -21,6 +20,7 @@
 #include "Client.hh" // For isSkip()
 #include "WindowManager.hh"
 #include "WorkspaceIndicator.hh"
+#include "x11.hh"
 
 #include <iostream>
 #include <sstream>
@@ -55,46 +55,20 @@ Workspaces::Workspace::~Workspace(void)
 
 // Workspaces
 
-Workspaces *Workspaces::_instance = 0;
-
-
-//! @brief Workspaces constructor
-Workspaces::Workspaces(uint number, uint per_row)
-  : _active(0), _previous(0), _per_row(per_row)
-{
-#ifdef DEBUG
-    if (_instance) {
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "Workspaces(" << this << ")::Workspaces(" << number << ")"
-             << endl << " *** _instance already set: " << _instance << endl;
-    }
-#endif // DEBUG
-    _instance = this;
-
-    if (number < 1) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "Workspaces(" << this << ")::Workspaces(" << number << ")"
-             << " *** number < 1" << endl;
-#endif // DEBUG
-        number = 1;
-    }
-
-    // create new workspaces
-    for (uint i = 0; i < number; ++i) {
-        _workspace_list.push_back(new Workspace(getWorkspaceName(i), i));
-    }
-}
+uint Workspaces::_active;
+uint Workspaces::_previous;
+uint Workspaces::_per_row;
+std::list<PWinObj*> Workspaces::_wo_list;
+std::vector<Workspaces::Workspace *> Workspaces::_workspace_list;
 
 //! @brief Workspaces destructor
-Workspaces::~Workspaces(void)
+void
+Workspaces::free(void)
 {
     vector<Workspace*>::iterator it(_workspace_list.begin());
     for (; it != _workspace_list.end(); ++it) {
         delete *it;
     }
-
-    _instance = 0;
 }
 
 //! @brief Sets total amount of workspaces to number
@@ -138,8 +112,8 @@ Workspaces::setSize(uint number)
     }
 
     // Tell the rest of the world how many workspaces we have.
-    XChangeProperty(PScreen::instance()->getDpy(),
-                    PScreen::instance()->getRoot(),
+    XChangeProperty(X11::getDpy(),
+                    X11::getRoot(),
                     Atoms::getAtom(NET_NUMBER_OF_DESKTOPS),
                     XA_CARDINAL, 32, PropModeReplace,
                     (uchar *) &number, 1);
@@ -172,7 +146,7 @@ Workspaces::setWorkspace(uint num, bool focus)
         return;
     }
 
-    PScreen::instance()->grabServer();
+    X11::grabServer();
 
     PWinObj *wo = PWinObj::getFocusedPWinObj();
     // Make sure that sticky windows gets unfocused on workspace change,
@@ -191,12 +165,12 @@ Workspaces::setWorkspace(uint num, bool focus)
 
     // switch workspace
     hideAll(_active);
-    AtomUtil::setLong(PScreen::instance()->getRoot(),
+    AtomUtil::setLong(X11::getRoot(),
                       Atoms::getAtom(NET_CURRENT_DESKTOP),
                       num);
     unhideAll(num, focus);
 
-    PScreen::instance()->ungrabServer(true);
+    X11::ungrabServer(true);
 
     // Show workspace indicator if requested
     if (Config::instance()->getShowWorkspaceIndicator() > 0) {
@@ -304,18 +278,18 @@ Workspaces::warpToWorkspace(uint num, int dir)
     }
 
     int x, y;
-    PScreen::instance()->getMousePosition(x, y);
+    X11::getMousePosition(x, y);
 
     if (dir != 0) {
       switch(dir) {
       case 1:
-        x = PScreen::instance()->getWidth() - std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_LEFT) + 2, 2);
+        x = X11::getWidth() - std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_LEFT) + 2, 2);
         break;
       case 2:
         x = std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_RIGHT) * 2, 2);
         break;
       case -1:
-        y = PScreen::instance()->getHeight() - std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) + 2, 2);
+        y = X11::getHeight() - std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) + 2, 2);
         break;
       case -2:
         y = std::max(Config::instance()->getScreenEdgeSize(SCREEN_EDGE_TOP) + 2, 2);
@@ -323,8 +297,8 @@ Workspaces::warpToWorkspace(uint num, int dir)
       }
 
         // warp pointer
-        XWarpPointer(PScreen::instance()->getDpy(), None,
-                     PScreen::instance()->getRoot(), 0, 0, 0, 0, x, y);
+        XWarpPointer(X11::getDpy(), None,
+                     X11::getRoot(), 0, 0, 0, 0, x, y);
     }
 
     // set workpsace
@@ -360,7 +334,7 @@ Workspaces::insert(PWinObj *wo, bool raise)
     _wo_list.insert(position, wo);
 
     if (position == _wo_list.end()) {
-         XRaiseWindow(PScreen::instance()->getDpy(), wo->getWindow());
+         XRaiseWindow(X11::getDpy(), wo->getWindow());
     } else {
          stackWinUnderWin((*position)->getWindow(), wo->getWindow());
     }
@@ -436,6 +410,12 @@ Workspaces::unhideAll(uint workspace, bool focus)
             // Focus
             wo->giveInputFocus();
             PWinObj::setFocusedPWinObj(wo);
+
+            // update the MRU list
+            if (wo->getType() == PWinObj::WO_CLIENT) {
+                Frame *frame = static_cast<Frame*>(wo->getParent());
+                WindowManager::instance()->addToFrameListBack(frame);
+            }
         }
 
         // If focusing fails, focus the root window.
@@ -558,7 +538,7 @@ Workspaces::stackWinUnderWin(Window over, Window under)
     }
 
     Window windows[2] = { over, under };
-    XRestackWindows(PScreen::instance()->getDpy(), windows, 2);
+    XRestackWindows(X11::getDpy(), windows, 2);
 }
 
 //! @brief Create name for workspace num
@@ -644,7 +624,7 @@ Workspaces::updateClientList(void)
     unsigned int num_windows;
     Window *windows = buildClientList(num_windows);
 
-    AtomUtil::setWindows(PScreen::instance()->getRoot(),
+    AtomUtil::setWindows(X11::getRoot(),
 			 Atoms::getAtom(NET_CLIENT_LIST),
 			 windows, num_windows);
     
@@ -660,7 +640,7 @@ Workspaces::updateClientStackingList(void)
     unsigned int num_windows;
     Window *windows = buildClientList(num_windows);
 
-    AtomUtil::setWindows(PScreen::instance()->getRoot(),
+    AtomUtil::setWindows(X11::getRoot(),
 			 Atoms::getAtom(NET_CLIENT_LIST_STACKING),
 			 windows, num_windows);
 
@@ -748,8 +728,7 @@ Workspaces::placeSmart(PWinObj* wo)
     uint wo_height = wo->getHeight() + Config::instance()->getPlacementOffsetY();
 
     Geometry head;
-    PScreen::instance()->getHeadInfoWithEdge(PScreen::instance()->getCurrHead(),
-            head);
+    X11::getHeadInfoWithEdge(X11::getCurrHead(), head);
 
     start_x = (Config::instance()->getPlacementLtR())
               ? (head.x)
@@ -811,11 +790,10 @@ bool
 Workspaces::placeMouseNotUnder(PWinObj *wo)
 {
     Geometry head;
-    PScreen::instance()->getHeadInfoWithEdge(PScreen::instance()->getCurrHead(),
-            head);
+    X11::getHeadInfoWithEdge(X11::getCurrHead(), head);
 
     int mouse_x, mouse_y;
-    PScreen::instance()->getMousePosition(mouse_x, mouse_y);
+    X11::getMousePosition(mouse_x, mouse_y);
 
     // compensate for head offset
     mouse_x -= head.x;
@@ -850,7 +828,7 @@ bool
 Workspaces::placeMouseCentered(PWinObj *wo)
 {
     int mouse_x, mouse_y;
-    PScreen::instance()->getMousePosition(mouse_x, mouse_y);
+    X11::getMousePosition(mouse_x, mouse_y);
 
     Geometry gm(mouse_x - (wo->getWidth() / 2), mouse_y - (wo->getHeight() / 2),
                 wo->getWidth(), wo->getHeight());
@@ -868,7 +846,7 @@ bool
 Workspaces::placeMouseTopLeft(PWinObj *wo)
 {
     int mouse_x, mouse_y;
-    PScreen::instance()->getMousePosition(mouse_x, mouse_y);
+    X11::getMousePosition(mouse_x, mouse_y);
 
     Geometry gm(mouse_x, mouse_y, wo->getWidth(), wo->getHeight());
     placeInsideScreen(gm); // make sure it's within the screens border
@@ -905,9 +883,9 @@ Workspaces::placeInsideScreen(Geometry &gm, Strut *strut)
     // along the edge of the screen.
     Geometry head;
     if (strut) {
-        PScreen::instance()->getHeadInfo(PScreen::instance()->getCurrHead(), head);
+        X11::getHeadInfo(X11::getCurrHead(), head);
     } else {
-        PScreen::instance()->getHeadInfoWithEdge(PScreen::instance()->getCurrHead(), head);
+        X11::getHeadInfoWithEdge(X11::getCurrHead(), head);
     }
 
     if (gm.x < head.x) {
@@ -1035,14 +1013,14 @@ Workspaces::findDirectional(PWinObj *wo, DirectionType dir, uint skip)
 
         if ((dir == DIRECTION_UP) || (dir == DIRECTION_DOWN)) {
             if ((wo_sec < (*it)->getX()) || (wo_sec > (*it)->getRX())) {
-                score += PScreen::instance()->getHeight() / 2;
+                score += X11::getHeight() / 2;
             }
             score += abs (static_cast<long> (wo_sec - ((*it)->getX ()
                                              + (*it)->getWidth () / 2)));
 
         } else {
             if ((wo_sec < (*it)->getY()) || (wo_sec > (*it)->getBY())) {
-                score += PScreen::instance()->getWidth() / 2;
+                score += X11::getWidth() / 2;
             }
 
             score += abs (static_cast<long> (wo_sec - ((*it)->getY ()
